dnl
dnl configure.in helper macros
dnl 

AC_DEFUN([WITH_PHP],[

  AC_ARG_WITH([php],
    [AS_HELP_STRING([--with-php],
      [Build NDB/PHP @<:@default=no@:>@])],
    [with_php=$withval],
    [with_php=no])

  AS_IF([test "x$with_php" != "xno"],[
    AS_IF([test "x$with_php" != "xyes"],
      [ac_check_php_config=$with_php],
      [ac_check_php_config="php-config php-config5"])
      AC_CHECK_PROGS(PHP_CONFIG, [$ac_check_php_config])
    ])

  AS_IF([test "x$PHP_CONFIG" != "x"],[
    PHP_CFLAGS=`$PHP_CONFIG --includes`
    PHP_CPPFLAGS=`$PHP_CONFIG --includes`
    PHP_LDFLAGS=`$PHP_CONFIG --ldflags`
    PHP_EXTDIR=`$PHP_CONFIG --extension-dir`
  ],[
    PHP_CFLAGS=
    PHP_CPPFLAGS=
    PHP_LDFLAGS=
    PHP_EXTDIR=
    with_php=no
  ])

  AC_SUBST(PHP_CFLAGS)
  AC_SUBST(PHP_CPPFLAGS)
  AC_SUBST(PHP_LDFLAGS)
  AC_SUBST(PHP_EXTDIR)
])

AC_DEFUN([WITH_PYTHON], [

  AC_ARG_WITH([python], 
    [AS_HELP_STRING([--with-python],
      [Build NDB/Python @<:@default=yes@:>@])],
    [with_python=$withval], 
    [with_python=python])

  AS_IF([test "x$with_python" != "xno"],[
    AS_IF([test "x$with_python" != "xyes"],[PYTHON=$with_python])
    AM_PATH_PYTHON([2.4],,[with_python="no"])
    AC_PYTHON_DEVEL()
    AS_IF([test "x$pythonexists" = "xno"],[with_python="no"])
  ])
])


AC_DEFUN([WITH_JAVA], [

  AC_ARG_WITH([java], 
    [AS_HELP_STRING([--with-java],
      [BULD NDB/J @<:@default=yes@:>@])],
    [with_java=$withval], 
    [with_java=java])

  AS_IF([test "x$with_java" != "xno"],[
    AS_IF([test "x$with_java" != "xyes"],[JAVA=$with_java])   
    AC_PROG_JAVA()
    AS_IF([test "x$JAVA" = "x"],AC_MSG_ERROR([Couldn't find java.]))
    AC_PROG_JAVAC()
    AS_IF([test "x$JAVAC" = "x"],AC_MSG_ERROR([Couldn't find javac.]))
    AC_PROG_JAR()
    AS_IF([test "x$JAR" = "x"],AC_MSG_ERROR([Couldn't find jar.]))
    AC_CHECK_PROGS(ANT,ant)
    AS_IF([test "x$ANT" = "x"],AC_MSG_ERROR([Couldn't find ant.]))
    AC_CHECK_JUNIT()
    AC_CHECK_JAVA_CXXFLAGS()
  ])
])

AC_DEFUN([WITH_CSHARP], [

  AC_ARG_WITH([csharp], 
    [AS_HELP_STRING([--with-csharp],
      [BULD NDB/.NET @<:@default=no@:>@])],
    [with_csharp=$withval], 
    [with_csharp=no])

  AS_IF([test "x$with_csharp" = "xyes"],[
    AC_CHECK_PROGS(MONO,[mono])
    AS_IF([test "x$MONO" = "x"],AC_MSG_ERROR([Couldn't find mono.]))
    AC_CHECK_PROGS(MCS,[gmcs])
    AS_IF([test "x$MCS" = "x"],AC_MSG_ERROR([Couldn't find gmcs.]))
  ])
])

AC_DEFUN([WITH_PERL], [

  AC_ARG_WITH([perl], 
    [AS_HELP_STRING([--with-perl],
      [BULD NDB/Perl @<:@default=yes@:>@])],
    [with_perl=$withval], 
    [with_perl=no])


  AS_IF([test "x$with_perl" != "xno"],[
    AS_IF([test "x$with_perl" != "xyes"],
      [ac_chk_perl=$with_perl],
      [ac_chk_perl=perl])
    AC_CHECK_PROGS(PERL,$ac_chk_perl)
  ])

dnl Don't think we need these anymore, but it's a good reference
dnl if test "x$PERL" != "x"; then
dnl  PERLCCFLAGS=`$PERL -MConfig -e 'print $Config{ccflags};'`
dnl  PERLCPPFLAGS=`$PERL -MConfig -e 'print $Config{cppflags};'`
dnl  PERLLIBS=`$PERL -MConfig -e 'print $Config{perllibs};'`
dnl fi 

])

AC_DEFUN([WITH_RUBY], [

  AC_ARG_WITH([ruby],
    [AS_HELP_STRING([--with-ruby],
      [BULD NDB/Ruby @<:@default=yes@:>@])],
      [with_ruby=$withval],
      [with_ruby=ruby])

  AS_IF([test "x$with_ruby" != "xno"],[
    AS_IF([test "x$with_ruby" != "xyes"],
      [ac_chk_ruby=$with_ruby],
      [ac_chk_ruby=ruby])
    AC_CHECK_PROGS(RUBY,$ac_chk_ruby)
  ])

  AS_IF([test "x$RUBY" != "x"],[

    AC_MSG_CHECKING(for ruby devel)

    dnl need to change quotes to allow square brackets
    changequote(<<, >>)dnl
    ruby_prefix=`$RUBY -rrbconfig -e "print Config::CONFIG['archdir']"`
    RUBY_LIB=`$RUBY -rrbconfig -e "puts Config::CONFIG['ruby_install_name']"`
    LIBRUBYARG_SHARED=`$RUBY -rrbconfig -e "puts Config::CONFIG['LIBRUBYARG_SHARED']"`
    RUBY_DIR=`$RUBY -rrbconfig -e "puts Config::CONFIG['archdir']"`
    RUBY_LIBDIR=`$RUBY -rrbconfig -e "puts Config::CONFIG['rubylibdir']"`
    RUBY_INCLUDES="-I$ruby_prefix"
    changequote([, ])dnl

    ac_save_CFLAGS="$CFLAGS"
    ac_save_LDFLAGS="$LDFLAGS"
    CFLAGS="$ac_save_CFLAGS $RUBY_INCLUDES"
    CPPFLAGS="$ac_save_CPPFLAGS $RUBY_INCLUDES"
    LDFLAGS="$ac_save_LDFLAGS $LIBRUBYARG_SHARED"
    
    AC_TRY_LINK([#include <ruby.h>], [VALUE rb_ac_test = rb_define_module("actest");],
                        [with_ruby="yes";AC_MSG_RESULT(found)],[with_ruby="no";AC_MSG_RESULT(missing)])

    CPPFLAGS="$ac_save_CPPFLAGS"
    LDFLAGS="$ac_save_LDFLAGS"
  ],[
    # This allows 'make clean' in the ruby directory to work when
    # ruby isn't available
    RUBY=
    RUBY_INCLUDES=
    LIBRUBYARG_SHARED=
    RUBY_LIB=
    RUBY_DIR=
    RUBY_LIBDIR=
    with_ruby="no"
  ])

  AC_SUBST(RUBY_INCLUDES)
  AC_SUBST(LIBRUBYARG_SHARED)
  AC_SUBST(RUBY_LIB)
  AC_SUBST(RUBY_DIR)
  AC_SUBST(RUBY_LIBDIR)

  AS_IF([test "x$RUBY_DIR" = "x"],[with_ruby="no"])
])
 
AC_DEFUN([WITH_MYSQL], [ 

  AC_MSG_CHECKING(for mysql_config executable)

  AC_ARG_WITH([mysql],
    [AS_HELP_STRING([--with-mysql=PATH],
        [path to mysql_config binary or mysql prefix dir])], 
      [with_mysql=$withval],
      [with_mysql=mysql_config])

   AS_IF([test -f $with_mysql -a -x $with_mysql],
      [MYSQL_CONFIG=$with_mysql],
      [AS_IF([test -f $with_mysql/bin/mysql_config -a -x $with_mysql/bin/mysql_config],
        [MYSQL_CONFIG=$with_mysql/bin/mysql_config])])
        
  AS_IF([test "x$with_mysql" = "xmysql_config"],
    AC_CHECK_PROGS(MYSQL_CONFIG,[$with_mysql]))

  AS_IF([test "x$MYSQL_CONFIG" = "x"],
    AC_MSG_ERROR([MySQL not found]))


    AC_PROG_CC
    AC_PROG_CXX
 
    AC_LANG(C++)

    # add regular MySQL C flags
    ADDCFLAGS=`$MYSQL_CONFIG --cflags` 
    # add NdbAPI specific C flags
    IBASE=`$MYSQL_CONFIG --include`
    ADDIFLAGS="$IBASE/storage/ndb"
    ADDIFLAGS="$ADDIFLAGS $IBASE/storage/ndb/ndbapi"
    ADDIFLAGS="$ADDIFLAGS $IBASE/storage/ndb/mgmapi"
    ADDIFLAGS="$ADDIFLAGS $IBASE/ndb"
    ADDIFLAGS="$ADDIFLAGS $IBASE/ndb/ndbapi"
    ADDIFLAGS="$ADDIFLAGS $IBASE/ndb/mgmapi"
    ADDIFLAGS="$ADDIFLAGS $IBASE"

    CFLAGS="$CFLAGS $ADDFLAGS $ADDIFLAGS"    
    CXXFLAGS="$CXXFLAGS $ADDFLAGS $ADDIFLAGS" 
    MYSQL_INCLUDES="$IBASE $ADDIFLAGS"   

    LDFLAGS="$LDFLAGS "`$MYSQL_CONFIG --libs_r`
    
    AC_CHECK_LIB([ndbhelper],[my_malloc],,[
      AC_CHECK_LIB([mysys],[my_malloc])
      AC_CHECK_LIB([mystrings],[my_str_free])
    ])
    AC_CHECK_LIB([ndbclient],[ndb_init],,[AC_MSG_ERROR([Can't link against libndbclient])])
     
    AC_MSG_CHECKING(for NdbApi headers)
     AC_TRY_LINK([#include <NdbApi.hpp>], [int attr=NdbTransaction::Commit; ],
                        [ndbapi_found="yes"])
    AS_IF([test "$ndbapi_found" = "yes"], 
       [AC_MSG_RESULT(found)],
       [AC_MSG_ERROR([Couldn't find NdbApi.h!])])

    LDFLAGS="$LDFLAGS $LIBS"
  

    MYSQL_MAJOR_VERSION=`$MYSQL_CONFIG --version | sed -e 's/\.//g' -e 's/-//g' -e 's/[A-Za-z]//g' | cut -c1-2`

    case "$MYSQL_MAJOR_VERSION" in 
      50) AC_DEFINE(MYSQL_50, [1], [mysql5.0])
	;;
      51) AC_DEFINE(MYSQL_51, [1], [mysql5.1])
        ;;
      *) echo "Unsupported version of MySQL Detected!"
        ;;
     esac
    
    AC_SUBST(MYSQL_MAJOR_VERSION)
    AC_SUBST(MYSQL_CONFIG)
    
  
])

