// -*- mode: c++ -*- 
/*  ndb-connectors: Wrappers for the NDBAPI
    Copyright (C) 2006 MySQL, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/** Log event specific data for for corresponding NDB_LE_ log event */

  /**
   * The NdbLogevent
   */
%rename ndb_logevent NdbLogEvent; 

class ndb_logevent { 

  /** NdbLogEventHandle (to be used for comparing only)
   *  set in ndb_logevent_get_next()
   */ 
  void *handle;

  /** Which event */
  enum Ndb_logevent_type type;

  /** Time when log event was registred at the management server */
  unsigned time;

  /** Category of log event */
  enum ndb_mgm_event_category category;

  /** Severity of log event */
  enum ndb_mgm_event_severity severity;

  /** Level (0-15) of log event */
  unsigned level;

  /** Node ID of the node that reported the log event */
  unsigned source_nodeid;

};

%extend ndb_logevent { 

public: 

  Ndb_logevent_type getEventType() { 
    return $self->type;
  }

  ndb_mgm_event_category getEventCategory() { 
    return $self->category; 
  }

  ndb_mgm_event_severity getEventSeverity() { 
    return $self->severity; 
  }
  
  Uint32 getSourceNodeId() { 
    return $self->source_nodeid;
  }

  Uint32 getEventTime() { 
    return $self->time;
  }

  Uint32 getEventLevel() { 
    return $self->level;
  }
  
};


%inline %{


  class BaseEvent { 
  protected: 
    BaseEvent()  { 
//      event=NULL; 
    }
  public: 
    ndb_logevent event; 
    BaseEvent(ndb_logevent theEvent) { 
      event=theEvent;
    } 
    Ndb_logevent_type getType() { 
      return event.type;
    }
    int getSourceNodeId() { 
      return event.source_nodeid;
    }
  };



  class ConnectedEvent : public BaseEvent { 
  public:
    ConnectedEvent(ndb_logevent theEvent) { 
      event=theEvent; 
    }
    unsigned getNode() { 
      return event.Connected.node;
    }
  };
    
  class DisconnectedEvent : public BaseEvent { 
  public:
    DisconnectedEvent(ndb_logevent theEvent) { 
      event=theEvent; 
    }
    unsigned getNode() { 
      return event.Disconnected.node;
    }
  };

  class CommunicationOpenedEvent : public BaseEvent { 
  public:
    CommunicationOpenedEvent(ndb_logevent theEvent) { 
      event=theEvent; 
    }
    unsigned getNode() { 
      return event.CommunicationOpened.node;
    }
  };

  class CommunicationClosedEvent : public BaseEvent { 
  public:
    CommunicationClosedEvent(ndb_logevent theEvent) { 
      event=theEvent; 
    }
    unsigned getNode() { 
      return event.CommunicationClosed.node;
    }
  };
  class ConnectedApiVersionEvent : public BaseEvent { 
  public:
    ConnectedApiVersionEvent(ndb_logevent theEvent) { 
      event=theEvent; 
    }
    unsigned getNode() { 
      return event.ConnectedApiVersion.node;
    }
    unsigned getVersion() { 
      return event.ConnectedApiVersion.version;
    }
  };
  

  class GlobalCheckpointStartedEvent : public BaseEvent { 
  public:

    unsigned gci;

    GlobalCheckpointStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
      gci=event.GlobalCheckpointStarted.gci;
    }
  };

  class GlobalCheckpointCompletedEvent : public BaseEvent { 
  public:
    
    unsigned gci;

    GlobalCheckpointCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      gci=event.GlobalCheckpointCompleted.gci;
    }
  };


  class LocalCheckpointStartedEvent : public BaseEvent { 
  public:

    unsigned lci;
    unsigned keepGci;
    unsigned restoreGci;
    
    LocalCheckpointStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
      lci = event.LocalCheckpointStarted.lci;
      keepGci = event.LocalCheckpointStarted.keep_gci;
      restoreGci = event.LocalCheckpointStarted.restore_gci;
      
    }
  };

  class LocalCheckpointCompletedEvent : public BaseEvent { 
  public:
    unsigned lci;

    LocalCheckpointCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      lci=event.LocalCheckpointCompleted.lci;
    }
  };

  class LCPStoppedInCalcKeepGciEvent : public BaseEvent { 
  public:
    unsigned data;

    LCPStoppedInCalcKeepGciEvent(ndb_logevent theEvent) {
      event=theEvent;
      data=event.LCPStoppedInCalcKeepGci.data;
    }
  };

  class LCPFragmentCompletedEvent : public BaseEvent { 
  public:
    unsigned node;
    unsigned tableId;
    unsigned fragmentId;

    LCPFragmentCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      node=event.LCPFragmentCompleted.node;
      tableId=event.LCPFragmentCompleted.table_id;
      fragmentId=event.LCPFragmentCompleted.fragment_id;
    }
  };

  class UndoLogBlockedEvent : public BaseEvent { 
  public:
    unsigned accCount;
    unsigned tupCount;
    
    UndoLogBlockedEvent(ndb_logevent theEvent) {
      event=theEvent;
      accCount=event.UndoLogBlocked.acc_count;
      tupCount=event.UndoLogBlocked.tup_count;
    }
  };

  class NDBStartStartedEvent : public BaseEvent { 
  public:
    unsigned version;

    NDBStartStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
      version=event.NDBStartStarted.version;
    }
  };

  class NDBStartCompletedEvent : public BaseEvent { 
  public:
    unsigned version;

    NDBStartCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      version=event.NDBStartCompleted.version;
    }
  };

  class STTORRYRecievedEvent : public BaseEvent { 
  public:
    STTORRYRecievedEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class StartPhaseCompletedEvent : public BaseEvent { 
  public:
    unsigned phase;
    unsigned startType;

    StartPhaseCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      phase=event.StartPhaseCompleted.phase;
      startType=event.StartPhaseCompleted.starttype;
    }
  };

  class CmRegConfEvent : public BaseEvent { 
  public:
    unsigned ownId;
    unsigned presidentId;
    unsigned dynamicId;

    CmRegConfEvent(ndb_logevent theEvent) {
      event=theEvent;
      ownId=event.CM_REGCONF.own_id;
      presidentId=event.CM_REGCONF.president_id;
      dynamicId=event.CM_REGCONF.dynamic_id;
    }
  };

  class CmRegRefEvent : public BaseEvent { 
  public:
    unsigned ownId;
    unsigned otherId;
    unsigned cause;

    CmRegRefEvent(ndb_logevent theEvent) {
      event=theEvent;
      ownId=event.CM_REGREF.own_id;
      otherId=event.CM_REGREF.other_id;
      cause=event.CM_REGREF.cause;
    }
  };

  class FindNeighboursEvent : public BaseEvent { 
  public:
    unsigned ownId;
    unsigned leftId;
    unsigned rightId;
    unsigned dynamicId;

    FindNeighboursEvent(ndb_logevent theEvent) {
      event=theEvent;
      ownId=event.FIND_NEIGHBOURS.own_id;
      leftId=event.FIND_NEIGHBOURS.left_id;
      rightId=event.FIND_NEIGHBOURS.right_id;
      dynamicId=event.FIND_NEIGHBOURS.dynamic_id;
    }
  };

  class NDBStopStartedEvent : public BaseEvent { 
  public:
    unsigned stopType;
    NDBStopStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
      stopType=event.NDBStopStarted.stoptype;
    }
  };

  class NDBStopCompletedEvent : public BaseEvent { 
  public:
    unsigned action;
    unsigned sigNum;

    NDBStopCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      action=event.NDBStopCompleted.action;
      sigNum=event.NDBStopCompleted.signum;
    }
  };

  class NDBStopForcedEvent : public BaseEvent { 
  public:
    unsigned action;
    unsigned signum;
    unsigned error;
    unsigned sphase;
    unsigned extra;
    
    NDBStopForcedEvent(ndb_logevent theEvent) {
      event=theEvent;
      action=event.NDBStopForced.action;
      signum=event.NDBStopForced.signum;
      error=event.NDBStopForced.error;
      sphase=event.NDBStopForced.sphase;
      extra=event.NDBStopForced.extra;
    }
  };

  class NDBStopAbortedEvent : public BaseEvent { 
  public:
    NDBStopAbortedEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class StartREDOLogEvent : public BaseEvent { 
  public:
    unsigned node;
    unsigned keepGci;
    unsigned completedGci;
    unsigned restorableGci;
    
    StartREDOLogEvent(ndb_logevent theEvent) {
      event=theEvent;
      node=event.StartREDOLog.node;
      keepGci=event.StartREDOLog.keep_gci;
      completedGci=event.StartREDOLog.completed_gci;
      restorableGci=event.StartREDOLog.restorable_gci;
    }
  };

  class StartLogEvent : public BaseEvent { 
  public:
    unsigned logPart;
    unsigned startMb;
    unsigned stopMb;
    unsigned gci;

    StartLogEvent(ndb_logevent theEvent) {
      event=theEvent;

      logPart=event.StartLog.log_part;
      startMb=event.StartLog.start_mb;
      stopMb=event.StartLog.stop_mb;
      gci=event.StartLog.gci;
    }
  };

  class UNDORecordsExecutedEvent : public BaseEvent { 
  public:
    unsigned block;
    unsigned data1;
    unsigned data2;
    unsigned data3;
    unsigned data4;
    unsigned data5;
    unsigned data6;
    unsigned data7;
    unsigned data8;
    unsigned data9;
    unsigned data10;
    UNDORecordsExecutedEvent(ndb_logevent theEvent) {
      event=theEvent;
      block=event.UNDORecordsExecuted.block;
      data1=event.UNDORecordsExecuted.data1;
      data2=event.UNDORecordsExecuted.data2;
      data3=event.UNDORecordsExecuted.data3;
      data4=event.UNDORecordsExecuted.data4;
      data5=event.UNDORecordsExecuted.data5;
      data6=event.UNDORecordsExecuted.data6;
      data7=event.UNDORecordsExecuted.data7;
      data8=event.UNDORecordsExecuted.data8;
      data9=event.UNDORecordsExecuted.data9;
      data10=event.UNDORecordsExecuted.data10;
    }
  };

  class NRCopyDictEvent : public BaseEvent { 
  public:
    NRCopyDictEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class NRCopyDistrEvent : public BaseEvent { 
  public:
    NRCopyDistrEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class NRCopyFragsStartedEvent : public BaseEvent { 
  public:
    unsigned destNode;

    NRCopyFragsStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
      destNode=event.NR_CopyFragsStarted.dest_node;
    }


  };

  class NRCopyFragDoneEvent : public BaseEvent { 
  public:

    unsigned destNode;
    unsigned tableId;
    unsigned fragmentId;

    NRCopyFragDoneEvent(ndb_logevent theEvent) {
      event=theEvent;
      destNode=event.NR_CopyFragDone.dest_node;
      tableId=event.NR_CopyFragDone.table_id;
      fragmentId=event.NR_CopyFragDone.fragment_id;
    }


  };

  class NRCopyFragsCompletedEvent : public BaseEvent { 
  public:
    unsigned destNode;
    NRCopyFragsCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      destNode=event.NR_CopyFragsCompleted.dest_node;
    }

  };

  class NodeFailCompletedEvent : public BaseEvent { 
  public:
    unsigned block; /* 0 = all */
    unsigned failedNode;
    unsigned completingNode; /* 0 = all */
    NodeFailCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      block=event.NodeFailCompleted.block;
      failedNode=event.NodeFailCompleted.failed_node;
      completingNode=event.NodeFailCompleted.completing_node; 
    }

  };

  class NodeFAILREPEvent : public BaseEvent { 
  public:
    unsigned failedNode;
    unsigned failureState;
    NodeFAILREPEvent(ndb_logevent theEvent) {
      event=theEvent;
      failedNode=event.NODE_FAILREP.failed_node;
      failureState=event.NODE_FAILREP.failure_state;
    }

  };

  class ArbitStateEvent : public BaseEvent { 
  public:

    unsigned code;                /* code & state << 16 */
    unsigned arbitNode;
    unsigned ticket0;
    unsigned ticket1;
    /* TODO */
    ArbitStateEvent(ndb_logevent theEvent) {
      event=theEvent;
      code=event.ArbitState.code;
      arbitNode=event.ArbitState.arbit_node;
      ticket0=event.ArbitState.ticket_0;
      ticket1=event.ArbitState.ticket_1;
    }

  };

  class ArbitResultEvent : public BaseEvent { 
  public:

    unsigned code;                /* code & state << 16 */
    unsigned arbitNode;
    unsigned ticket0;
    unsigned ticket1;
    ArbitResultEvent(ndb_logevent theEvent) {
      event=theEvent;
      code=event.ArbitResult.code; 
      arbitNode=event.ArbitResult.arbit_node;
      ticket0=event.ArbitResult.ticket_0;
      ticket1=event.ArbitResult.ticket_1;
    }

  };

  class GCPTakeoverStartedEvent : public BaseEvent { 
  public:
    GCPTakeoverStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class GCPTakeoverCompletedEvent : public BaseEvent { 
  public:
    GCPTakeoverCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class LCPTakeoverStartedEvent : public BaseEvent { 
  public:
    LCPTakeoverStartedEvent(ndb_logevent theEvent) {
      event=theEvent;
    }
  };

  class LCPTakeoverCompletedEvent : public BaseEvent { 
  public:
    unsigned state;
    LCPTakeoverCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;
      state=event.LCP_TakeoverCompleted.state;
    }
  };

  class TransReportCountersEvent : public BaseEvent { 
  public:
    unsigned transCount;
    unsigned commitCount;
    unsigned readCount;
    unsigned simpleReadCount;
    unsigned writeCount;
    unsigned attrinfoCount;
    unsigned concOpCount;
    unsigned abortCount;
    unsigned scanCount;
    unsigned rangeScanCount;
    TransReportCountersEvent(ndb_logevent theEvent) {
      event=theEvent;


      transCount=event.TransReportCounters.trans_count;
      commitCount=event.TransReportCounters.commit_count;
      readCount=event.TransReportCounters.read_count;
      simpleReadCount=event.TransReportCounters.simple_read_count;
      writeCount=event.TransReportCounters.write_count;
      attrinfoCount=event.TransReportCounters.attrinfo_count;
      concOpCount=event.TransReportCounters.conc_op_count;
      abortCount=event.TransReportCounters.abort_count;
      scanCount=event.TransReportCounters.scan_count;
      rangeScanCount=event.TransReportCounters.range_scan_count;
    }

  };

  class OperationReportCountersEvent : public BaseEvent { 
  public:

    unsigned ops;
    OperationReportCountersEvent(ndb_logevent theEvent) {
      event=theEvent;


      ops=event.OperationReportCounters.ops;
    }

  };

  class TableCreatedEvent : public BaseEvent { 
  public:

    unsigned tableId;
    TableCreatedEvent(ndb_logevent theEvent) {
      event=theEvent;


      tableId=event.TableCreated.table_id;
    }

  };

  class JobStatisticEvent : public BaseEvent { 
  public:

    unsigned meanLoopCount;
    JobStatisticEvent(ndb_logevent theEvent) {
      event=theEvent;


      meanLoopCount=event.JobStatistic.mean_loop_count;
    }

  };

  class SendBytesStatisticEvent : public BaseEvent { 
  public:

    unsigned toNode;
    unsigned meanSentBytes;
    SendBytesStatisticEvent(ndb_logevent theEvent) {
      event=theEvent;


      toNode=event.SendBytesStatistic.to_node;
      meanSentBytes=event.SendBytesStatistic.mean_sent_bytes;
    }

  };

  class ReceiveBytesStatisticEvent : public BaseEvent { 
  public:

    unsigned fromNode;
    unsigned meanReceivedBytes;
    ReceiveBytesStatisticEvent(ndb_logevent theEvent) {
      event=theEvent;


      fromNode=event.ReceiveBytesStatistic.from_node;
      meanReceivedBytes=event.ReceiveBytesStatistic.mean_received_bytes;
    }

  };

  class MemoryUsageEvent : public BaseEvent { 
  public:

    int      gth;
    unsigned pageSizeKb;
    unsigned pagesUsed;
    unsigned pagesTotal;
    unsigned block;
    MemoryUsageEvent(ndb_logevent theEvent) {
      event=theEvent;


      gth=event.MemoryUsage.gth;
      pageSizeKb=event.MemoryUsage.page_size_kb;
      pagesUsed=event.MemoryUsage.pages_used;
      pagesTotal=event.MemoryUsage.pages_total;
      block=event.MemoryUsage.block;
    }

  };

  class TransporterErrorEvent : public BaseEvent { 
  public:


    unsigned toNode;
    unsigned code;
    TransporterErrorEvent(ndb_logevent theEvent) {
      event=theEvent;


      toNode=event.TransporterError.to_node;
      code=event.TransporterError.code;
    }

  };

  class TransporterWarningEvent : public BaseEvent { 
  public:

    unsigned toNode;
    unsigned code;
    TransporterWarningEvent(ndb_logevent theEvent) {
      event=theEvent;


      toNode=event.TransporterWarning.to_node;
      code=event.TransporterWarning.code;
    }

  };

  class MissedHeartbeatEvent : public BaseEvent { 
  public:

    unsigned node;
    unsigned count;
    MissedHeartbeatEvent(ndb_logevent theEvent) {
      event=theEvent;


      node=event.MissedHeartbeat.node;
      count=event.MissedHeartbeat.count;
    }

  };

  class DeadDueToHeartbeatEvent : public BaseEvent { 
  public:

    unsigned node;
    DeadDueToHeartbeatEvent(ndb_logevent theEvent) {
      event=theEvent;


      node=event.DeadDueToHeartbeat.node;
    }

  };

  class WarningEventEvent : public BaseEvent { 
  public:
    WarningEventEvent(ndb_logevent theEvent) {
      event=theEvent;
    }

    /* TODO */

  };

  class SentHeartbeatEvent : public BaseEvent { 
  public:

    unsigned node;
    SentHeartbeatEvent(ndb_logevent theEvent) {
      event=theEvent;


      node=event.SentHeartbeat.node;
    }

  };

  class CreateLogBytesEvent : public BaseEvent { 
  public:

    unsigned node;
    CreateLogBytesEvent(ndb_logevent theEvent) {
      event=theEvent;


      node=event.CreateLogBytes.node;
    }

  };

  class InfoEventEvent : public BaseEvent { 
  public:
    InfoEventEvent(ndb_logevent theEvent) {
      event=theEvent;
    }

    /* TODO */

  };

  class EventBufferStatusEvent : public BaseEvent { 
  public:

    unsigned usage;
    unsigned alloc;
    unsigned max;
    unsigned applyGciL;
    unsigned applyGciH;
    unsigned latestGciL;
    unsigned latestGciH;
    EventBufferStatusEvent(ndb_logevent theEvent) {
      event=theEvent;


      usage=event.EventBufferStatus.usage;
      alloc=event.EventBufferStatus.alloc;
      max=event.EventBufferStatus.max;
      applyGciL=event.EventBufferStatus.apply_gci_l;
      applyGciH=event.EventBufferStatus.apply_gci_h;
      latestGciL=event.EventBufferStatus.latest_gci_l;
      latestGciH=event.EventBufferStatus.latest_gci_h;
    }

  };


  class BackupStartedEvent : public BaseEvent { 
  public:

    unsigned startingNode;
    unsigned backupId;
    BackupStartedEvent(ndb_logevent theEvent) {
      event=theEvent;


      startingNode=event.BackupStarted.starting_node;
      backupId=event.BackupStarted.backup_id;
    }

  };

  class BackupFailedToStartEvent : public BaseEvent { 
  public:

    unsigned startingNode;
    unsigned error;
    BackupFailedToStartEvent(ndb_logevent theEvent) {
      event=theEvent;


      startingNode=event.BackupFailedToStart.starting_node;
      error=event.BackupFailedToStart.error;
    }

  };

  class BackupCompletedEvent : public BaseEvent { 
  public:

    unsigned startingNode;
    unsigned backupId; 
    unsigned startGci;
    unsigned stopGci;
    unsigned numRecords; 
    unsigned numLogRecords;
    unsigned numBytes;
    unsigned numLogBytes;
    BackupCompletedEvent(ndb_logevent theEvent) {
      event=theEvent;


      startingNode=event.BackupCompleted.starting_node;
      backupId=event.BackupCompleted.backup_id; 
      startGci=event.BackupCompleted.start_gci;
      stopGci=event.BackupCompleted.stop_gci;
      numRecords=event.BackupCompleted.n_records; 
      numLogRecords=event.BackupCompleted.n_log_records;
      numBytes=event.BackupCompleted.n_bytes;
      numLogBytes=event.BackupCompleted.n_log_bytes;
    }

  };

  class BackupAbortedEvent : public BaseEvent { 
  public:

    unsigned startingNode;
    unsigned backupId;
    unsigned error;
    BackupAbortedEvent(ndb_logevent theEvent) {
      event=theEvent;


      startingNode=event.BackupAborted.starting_node;
      backupId=event.BackupAborted.backup_id;
      error=event.BackupAborted.error;
    }

  };

  class SingleUserEvent : public BaseEvent { 
  public: 

    unsigned eventType;
    unsigned nodeId;
    SingleUserEvent(ndb_logevent theEvent) {
      event=theEvent;


      eventType=event.SingleUser.type;
      nodeId=event.SingleUser.node_id;
    }

  };

  class StartReportEvent : public BaseEvent { 
  public:


    unsigned reportType;
    unsigned remainingTime;
    unsigned bitmaskSize;
    //unsigned bitmaskData[1];
    StartReportEvent(ndb_logevent theEvent) {
      event=theEvent;


      reportType=event.StartReport.report_type;
      remainingTime=event.StartReport.remaining_time;
      bitmaskSize=event.StartReport.bitmask_size;
      //bitmaskData[0]=event.StartReport.bitmask_data[0];
    }

  };


  %}

%{

  struct BaseEventWrapper { 
    int ret; 
    ndb_logevent * theEvent; 
  };

  %}
