// -*- mode: c++ -*- 
/*  ndb-connectors: Wrappers for the NDBAPI
    Copyright (C) 2006 MySQL, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


%{
#include <vector>
#include <map>
#include <stdexcept>
  %}

%{ 
  class NdbLogEventListener { 
  public: 
    virtual ~NdbLogEventListener() {} 
    virtual void handleEvent(ndb_logevent * event) {}
  };
  class NdbLogEventCategoryListener { 
  public: 
    virtual ~NdbLogEventCategoryListener() {} 
    virtual void handleEvent(ndb_logevent * event) {}
    virtual ndb_mgm_event_category getEventCategory() { return NDB_MGM_ILLEGAL_EVENT_CATEGORY; }
  };
  class NdbLogEventTypeListener { 
  public: 
    virtual ~NdbLogEventTypeListener() {} 
    virtual void handleEvent(ndb_logevent * event) {}
    virtual Ndb_logevent_type getEventType() { return NDB_LE_ILLEGAL_TYPE; } 
  };
  
  class NdbLogEventManager { 
    
    ndb_logevent_handle * handle; 
    
    std::map<Ndb_logevent_type,NdbLogEventTypeListener *> evtListeners; 
    std::map<ndb_mgm_event_category,NdbLogEventCategoryListener *> evtCategoryListeners;
    
  public: 
   
    NdbLogEventManager(ndb_logevent_handle * theHandle) { 
      this->handle=theHandle;
    }
   
    ~NdbLogEventManager() {
      ndb_mgm_destroy_logevent_handle(&handle);
    }
    const char * getMgmError() { 
      return ndb_logevent_get_latest_error_msg(this->handle); 
    }

    // Returns -1 on error, 0 otherwise
    int registerListener(NdbLogEventTypeListener * listener) { 
      Ndb_logevent_type theType = listener->getEventType();
      
      
      evtListeners.insert(std::make_pair<Ndb_logevent_type,NdbLogEventTypeListener *>(theType,listener));
	
      
      return 0;
    }
    
    bool unregisterListener(NdbLogEventTypeListener * listener) { 
      return true;
    }
    // Returns -1 on error, 0 otherwise
    int registerCategoryListener(NdbLogEventCategoryListener * listener) { 
      ndb_mgm_event_category theCategory = listener->getEventCategory();
      
      
      NdbLogEventCategoryListener * evtLis; 

      
      try {
	evtLis = evtCategoryListeners.at(theCategory);
      } catch (std::out_of_range e) { 
	if (!((evtCategoryListeners.insert(std::make_pair<ndb_mgm_event_category,NdbLogEventCategoryListener*>(theCategory,evtLis))).second)) {
	  return -1;
	}
      }; 
      return 0;
    }
    
    
    BaseEventWrapper * getLogEvent(unsigned timeout_in_milliseconds) { 
      
      //TODO: replace malloc with a pinned byte-array, until get_next_event()
      // is called, then unpin it
      
      /*ndb_logevent* event = (ndb_logevent*) malloc (sizeof(ndb_logevent));
	if (event==0) { 
	return NULL; 
	}*/
      ndb_logevent * event = (ndb_logevent*) malloc (sizeof(ndb_logevent));
      if (event==0) { 
	return NULL; 
      }
      BaseEventWrapper * eventWrapper = (BaseEventWrapper *)malloc(sizeof(BaseEventWrapper)); 
      
      //int r= ndb_logevent_get_next(handle,&(*event),timeout_in_milliseconds);
      eventWrapper->ret = ndb_logevent_get_next(handle,&(*event),timeout_in_milliseconds);
      
      eventWrapper->theEvent = NULL;
      //TODO: Figure out how to deal with no event properly
      if (eventWrapper->ret > 0) { 
	eventWrapper->theEvent = event;
	
	NdbLogEventTypeListener * evtLis = NULL;
	NdbLogEventCategoryListener * catLis = NULL;
	
	try {
	  evtLis = evtListeners.at(event->type);
	  
	} catch (std::out_of_range e) { 
	  // We don't have a listener to fire
	}
	if (evtLis != NULL) { 
	  evtLis->handleEvent(event);
	}
	try {
	  catLis = evtCategoryListeners.at(event->category);
	  
	} catch (std::out_of_range e) { 
	  // We don't have a listener to fire
	}
	if (catLis != NULL) { 
	  catLis->handleEvent(event);
	}

	// r > 0, event exists; 
	// r==0, no event (timeout)
	// r==-1, error
	
      }    
      return eventWrapper; 
    
    }  
  };
    

    /**
     * Attempt to retrieve next log event and will fill in the supplied
     * struct dst
     *
     * @param dst Pointer to struct to fill in event information
     * @param timeout_in_milliseconds Timeout for waiting for event
     *
     * @return     >0 if event exists, 0 no event (timed out), or -1 on error.
     *
     * @note Return value <=0 will leave dst untouched
     */
/*  int getNext(ndb_logevent *dst,
    unsigned timeout_in_milliseconds) { 
    return ndb_logevent_get_next($self, dst, timeout_in_milliseconds);
    }*/
    

    %}


%feature("director") NdbLogEventTypeListener;
%feature("director") NdbLogEventCategoryListener;

class NdbLogEventListener { 
public: 
  virtual ~NdbLogEventListener(); 
  virtual void handleEvent(ndb_logevent * event);
};
class NdbLogEventCategoryListener { 
public: 
  virtual ~NdbLogEventCategoryListener();
  virtual void handleEvent(ndb_logevent * event);
  virtual ndb_mgm_event_category getEventCategory();
};
class NdbLogEventTypeListener { 
public: 
  virtual ~NdbLogEventTypeListener();
  virtual void handleEvent(ndb_logevent * event);
  virtual Ndb_logevent_type getEventType();
};

%typemap(newfree) (BaseEventWrapper *) "free($1);";
%newobject NdbLogEventManager::getLogEvent;

class NdbLogEventManager { 
  NdbLogEventManager();
  ndb_logevent_handle * handle; 
  NdbLogEventManager(ndb_logevent_handle * theHandle);
public: 
  // Returns -1 on error, 0 otherwise
  ~NdbLogEventManager();
  int registerListener(NdbLogEventTypeListener * listener);   
  bool unregisterListener(NdbLogEventTypeListener * listener);
  int registerCategoryListener(NdbLogEventCategoryListener * listener);

  %ndbexception("MgmApiException") { 
    $action
      if (result->ret < 0) { 
	NDB_exception(NdbApiException,arg1->getMgmError());
      }
  }
  BaseEventWrapper * getLogEvent(unsigned timeout_in_milliseconds);
  %ndbnoexception; 
};
  

  /**
   * The NdbMgmLogeventHandle.
   */


/**
 * The NdbLogEventHandle
 */

//%rename ndb_logevent_handle NdbLogEventManager; 
/*
class ndb_logevent_handle { 
  ndb_logevent_handle();

};


%extend ndb_logevent_handle { 
  

public: 

  ~ndb_logevent_handle() { 
    ndb_mgm_destroy_logevent_handle(&$self);
  }

  /**
   * Retrieve laterst error code
   *
   * @return     error code
   */
  /*int getLatestError() { 
    return ndb_logevent_get_latest_error($self);
    }*/

  /**
   * Retrieve latest error message
   *
   * @return     error message
   */
// We aren't going to expose this in this way
//  const char * getLatestErrorMsgndb_logevent_get_latest_error_msg(const NdbLogEventHandle);

  /**
   * Retrieve filedescriptor from NdbLogEventHandle.  May be used in
   * e.g. an application select() statement.
   *
   * @note Do not attemt to read from it, it will corrupt the parsing.
   *
   * @return       filedescriptor, -1 on failure.
   */
  // TODO: haven't figured out FD's yet
//  int ndb_logevent_get_fd(const NdbLogEventHandle);



/*
}; 
*/

