/**
 * This is the NDB/J API package.
 */
package com.mysql.cluster.ndbj;


/**
 * Indicates that a temporary (transient) error has occured in the cluster (e.g., node failure) 
 * and that if you wait for a short time (e.g., 1500ms), the error may have
 * gone away.
 * <br>Transactions that are aborted due to NdbApiTemporaryExceptions should be retried.
 *<br>
 * <code>
 *      boolean finished = false;<br>
 *      while (finished == false && numRetries-- > 0) {<br>
 *        try {<br>
 *        // Set up Transaction and create operations<br>
 *          trans.execute(NdbTransaction.ExecType.Commit, NdbTransaction.AbortOption.AbortOnError, true);<br>
 *          finished = true;<br> // transaction succeded, exit loop
 *        } <br>
 *        catch (NdbApiTemporaryException e) {<br>
 *          finished = false;<br> // transaction failed, retry if numRetries > 0
 *        }<br>
 *        catch (NdbApiUserAndPermanentException e) {<br>
 *          finished = true;<br> // transaction failed, do not retry 
 *        }<br>
 *      }<br>
 * </code>
 * @see NdbTransaction
 */
public class NdbApiTemporaryException extends NdbApiException {

	protected static final long serialVersionUID=1L;
	/**
	 * @param error
	 */
	public NdbApiTemporaryException(long error) {
		super(error);
	}

	/**
	 * @param message
	 */
	public NdbApiTemporaryException(String message) {
		super(message);
	}

	
}
