// -*- mode: c++ -*-
/*  ndb-connectors: Wrappers for the NDBAPI
    Copyright (C) 2006 MySQL, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

%module ndbj


%rename NdbBlob NdbBlobImpl;
%rename NdbClusterConnection NdbClusterConnectionImpl;
%rename NdbEventOperation NdbEventOperationImpl;
%rename NdbIndexOperation NdbIndexOperationImpl;
%rename NdbIndexScanOperation NdbIndexScanOperationImpl;
%rename NdbOperation NdbOperationImpl;
%rename NdbRecAttr NdbRecAttrImpl;
%rename Ndb NdbImpl;
%rename NdbScanFilter NdbScanFilterImpl;
%rename NdbScanOperation NdbScanOperationImpl;
%rename NdbTransaction NdbTransactionImpl;


%include "ndbapi/ndbglobals.i"
%include "various.i"
%include "enums.swg"


%{
#include <stdio.h>

typedef struct
asynch_callback_t
{  
  JNIEnv *env; 
  jobject obj;
  long long create_time;
};



static void theCallBack(int result,
                     NdbTransaction *trans,
                     void *aObject)
{

  asynch_callback_t * callback_data = (asynch_callback_t *)aObject;

  JNIEnv *jenv = callback_data->env;
//  printf("Running callback: env == %p .. callback_data->obj == %p\n",jenv,callback_data->obj); 

  jclass callablecls = jenv->GetObjectClass((callback_data->obj));

  // printf("\tcallablecls: %p\n",callablecls);
  jmethodID mid = jenv->GetMethodID(callablecls, "jni_call_callback","(IJJ)V");

  //jmethodID mid = jenv->GetMethodID(callablecls, "callback", "(ILNdbTransaction;)V");
  //printf("\tmid: %p\n",mid);
  if ((mid == NULL) || (mid == 0)) { 
    jenv->DeleteGlobalRef(callback_data->obj);
    jclass clazz = jenv->FindClass("java/lang/RuntimeException");
    jenv->ThrowNew(clazz,"No callback method!");
  }

  jenv->CallVoidMethod(callback_data->obj,mid,(jlong)result,trans,callback_data->create_time);
  jenv->DeleteGlobalRef(callback_data->obj);
  delete callback_data; // Clean up the handle containing the callback info
}


#define NDB_exception(excp, msg) { ndb_throw_exception(jenv, #excp,msg);  } 
#define NDB_exception_err(excp, msg, err) { ndb_throw_exception(jenv, #excp,msg, err);  } 

void ndb_throw_exception(JNIEnv *jenv, const char * excp, const char *msg) { 
  const char * prefix = "com/mysql/cluster/ndbj/";
  char * exception = (char *)malloc(strlen(excp)+strlen(prefix)+1);
  strcpy(exception,prefix);
  strcat(exception,excp);

  //jclass clazz = jenv->FindClass("java/lang/RuntimeException");
  jclass clazz = jenv->FindClass(exception);
  free(exception);

  jenv->ThrowNew(clazz,msg);
  jenv->DeleteLocalRef(clazz);
}

void ndb_throw_exception(JNIEnv *jenv, const char * excp, const char *msg, NdbError & err) { 
  const char * prefix = "com/mysql/cluster/ndbj/";
  char * exception = (char *)malloc(strlen(excp)+strlen(prefix)+1);
  strcpy(exception,prefix);
  strcat(exception,excp);

  //jclass clazz = jenv->FindClass("java/lang/RuntimeException");
  jclass clazz = jenv->FindClass(exception);
  free(exception);

  jlong errPtr = 0; 
  NdbError * result = new NdbError(err);
  errPtr = (jlong)result;
  
  jmethodID method = jenv->GetMethodID(clazz,"<init>","(J)V");
  jthrowable excpObj = (jthrowable)jenv->NewObject(clazz,method,errPtr);

  jenv->Throw(excpObj);

  jenv->DeleteLocalRef(clazz);
  jenv->DeleteLocalRef(excpObj);
}

 %}


%typemap(in) asynch_callback_t * cb
%{ 
    $1 = new asynch_callback_t;
    $1->env = jenv; 
    $1->obj = jenv->NewGlobalRef($input);
    $1->create_time = 0;
//    jclass callablecls = jenv->GetObjectClass(($1->obj));    
//    printf("Registering callback: env==%p .. obj==%p .. callbackcls==%p\n",jenv,$1->obj,callablecls);
 %}
/*
%typemap(freearg) asynch_callback_t * cb
%{
  delete $1; 
  %}
*/
%typemap(in) event_callback_t * cb
%{ 
  
    callback_data = new event_callback_t;
    callback_data->env = jenv; 
    callback_data->obj = $input;
    
    $1 = callback_data;
 %}


%typemap(jni) asynch_callback_t * "jobject" 
%typemap(jtype) asynch_callback_t * "Object"
%typemap(jstype) asynch_callback_t * "BaseCallback"
%typemap(javain) asynch_callback_t * "(Object)$javainput"

//We're replacing this return val with an exception
%typemap(out) voidint ""; 
%typemap(jtype) voidint "void";
%typemap(jstype) voidint "void";
%typemap(jni) voidint "void";
%typemap(javaout) voidint { 
  $jnicall; 
}


%typemap(jni) MYSQL_TIME * "jobject"
%typemap(jtype) MYSQL_TIME * "Object"
%typemap(jstype) MYSQL_TIME * "java.util.Calendar"
%typemap(javain) MYSQL_TIME * "$javainput"
%typemap(javaout) MYSQL_TIME * { return (java.util.Calendar)$jnicall; }

%typemap(jstype) NdbTimestamp "java.sql.Timestamp"
 //%typemap(jtype) NdbTimestamp "java.sql.Timestamp"
%typemap(javain) NdbTimestamp "$javainput.getTime()/1000"
%typemap(javaout) NdbTimestamp { return new java.sql.Timestamp($jnicall); }


%typemap(jni) BYTES "jbyteArray"
%typemap(jtype) BYTES "byte[]"
%typemap(jstype) BYTES "byte[]"
%typemap(javaout) BYTES { return $jnicall; } 

%typemap(in) (MYSQL_TIME *) { 
   MYSQL_TIME * dt = (MYSQL_TIME *)malloc(sizeof(MYSQL_TIME));
   static jclass calendar_class = jenv->FindClass("java/util/Calendar");
   static jmethodID getID = jenv->GetMethodID(calendar_class,"get","(I)I");
   static jfieldID yearID = jenv->GetFieldID(calendar_class,"YEAR","I");
   static jfieldID monthID = jenv->GetFieldID(calendar_class,"MONTH","I");
   static jfieldID dayID = jenv->GetFieldID(calendar_class,"DAY","I");
   static jfieldID hourID = jenv->GetFieldID(calendar_class,"HOUR","I");
   static jfieldID minuteID = jenv->GetFieldID(calendar_class,"MINUTE","I");
   static jfieldID secondID = jenv->GetFieldID(calendar_class,"SECOND","I");
   static jint yearfield = jenv->GetIntField(calendar_class,yearID);
   static jint monthfield = jenv->GetIntField(calendar_class,monthID);
   static jint dayfield = jenv->GetIntField(calendar_class,dayID);
   static jint hourfield = jenv->GetIntField(calendar_class,hourID);
   static jint minutefield = jenv->GetIntField(calendar_class,minuteID);
   static jint secondfield = jenv->GetIntField(calendar_class,secondID);
   jint yearval = jenv->CallIntMethod($input,getID,yearfield);
   jint monthval = jenv->CallIntMethod($input,getID,monthfield);
   jint dayval = jenv->CallIntMethod($input,getID,dayfield);
   jint hourval = jenv->CallIntMethod($input,getID,hourfield);
   jint minuteval = jenv->CallIntMethod($input,getID,minutefield);
   jint secondval = jenv->CallIntMethod($input,getID,secondfield);
   dt->year = (int)yearval;
   dt->month = (int)monthval;
   dt->day = (int)dayval;
   dt->hour = (int)hourval;
   dt->minute = (int)minuteval;
   dt->second = (int)secondval;
   $1 = dt;
 }

%typemap(out) (MYSQL_TIME *) { 
  static jclass calendar_class = jenv->FindClass("java/util/Calendar");
  static jmethodID get_calendar_instance = jenv->GetStaticMethodID(calendar_class,"getInstance","()Ljava/util/Calendar;");
  static jmethodID setID = jenv->GetMethodID(calendar_class,"set","(IIIIII)V");
  $result = jenv->CallObjectMethod(calendar_class,get_calendar_instance);
  jenv->CallVoidMethod($result,setID,$1->year,$1->month,$1->day,$1->hour,$1->minute,$1->second);

 }

%typemap(out) (BYTES)  { 
 
 $result = jenv->NewByteArray($1.theLength); 
 jenv->SetByteArrayRegion($result,0,$1.theLength,(jbyte*) $1.theString);
 
} 

%typemap(freearg) (MYSQL_TIME *) {
  free((MYSQL_TIME *) $1);
 }


%typemap(in) (const char* anInputString, size_t len) {
  /* Check that we are getting a string */
  $1 = (char *)(jenv->GetStringUTFChars($input,0));
  $2 = jenv->GetStringUTFLength($input);
 }

%typemap(freearg) (const char * anInputString, size_t len) { 
  if ($1) { 
    jenv->ReleaseStringUTFChars($input,$1); 
  }
 }

%typemap(in) (const char* BYTE, size_t len) { 
  $1 = (char *)(jenv->GetByteArrayElements($input,0));
  $2 = jenv->GetArrayLength($input);
 }
%typemap(in) (Int64 anInputTimestamp) { 
  // Java timestamp is higher precision - so we cut it down
  $1 = (Int64) ($input / 1000);
 }


/* char ** typemap */

/* This tells SWIG to treat char ** as a special case when used as a parameter
   in a function call */
%typemap(in) (unsigned noOfNames, const char ** names) {
    Uint32 i = 0;
    $1 = jenv->GetArrayLength($input);
    $2 = (char **) malloc(($1+1)*sizeof(char *));
    /* make a copy of each string */
    for (i = 0; i<$1; i++) {
      jstring j_string = (jstring)(jenv->GetObjectArrayElement($input, i));
      const char * c_string = jenv->GetStringUTFChars(j_string, 0);
      $2[i] = (char *)malloc(strlen((c_string)+1)*sizeof(const char *));
      strcpy($2[i], c_string);
      jenv->ReleaseStringUTFChars(j_string, c_string);
      jenv->DeleteLocalRef(j_string);
    }
    $2[i] = 0;
 }

/* This cleans up the memory we malloc'd before the function call */
%typemap(freearg) (unsigned noOfNames, const char ** names)  {
  Uint32 i;
  for (i=0; i<$1-1; i++)
    free($2[i]);
  free($2);
 }

/* This allows a C function to return a char ** as a Java String array */
%typemap(out) char ** {
  Uint32 i;
  Uint32 len=0;
  jstring temp_string;
  const jclass clazz = jenv->FindClass("java/lang/String");
  
  while ($1[len]) len++;    
  jresult = jenv->NewObjectArray(len, clazz, NULL);
  /* exception checking omitted */
  
  for (i=0; i<len; i++) {
    temp_string = jenv->NewStringUTF(*result++);
    jenv->SetObjectArrayElement(jresult, i, temp_string);
    jenv->DeleteLocalRef(temp_string);
  }
}

/* These 3 typemaps tell SWIG what JNI and Java types to use */
%typemap(jni) (unsigned noOfNames, const char ** names) "jobjectArray"
%typemap(jtype) (unsigned noOfNames, const char ** names) "String[]"
%typemap(jstype)(unsigned noOfNames, const char ** names) "String[]"

/* These 2 typemaps handle the conversion of the jtype to jstype typemap type
   and visa versa */
%typemap(javain) (unsigned noOfNames, const char ** names) "$javainput"
%typemap(javaout) char ** {
    return $jnicall;
  }



%typemap(javacode) SWIGTYPE %{
  public boolean equals(Object obj) {
    boolean equal = false;
    if (obj instanceof $javaclassname)
      equal = ((($javaclassname)obj).swigCPtr == this.swigCPtr);
    return equal;
  }
  public int hashCode() {
     return (int)getCPtr(this);
  }
%}


%typemap(javainterfaces) Ndb_cluster_connection "NdbClusterConnection";
%javamethodmodifiers Ndb_cluster_connection::deleteAllNdbObjects "protected";
%typemap(javafinalize) Ndb_cluster_connection %{ 
protected void finalize() { 
  deleteAllNdbObjects();
  delete();
}
%}

%include "ndbapi/NdbClusterConnection.i"

%typemap(javainterfaces) Ndb "Ndb"; 

/*
%rename(realStartTransaction) Ndb::startTransaction; 
%javamethodmodifiers Ndb::startTransaction "protected" 
%typemap(javacode) Ndb %{

  public NdbTransactionImpl startTransaction(NdbTable table, String keyData, long keyLen) throws NdbApiException {
    NdbTransactionImpl theTrans  = realStartTransaction(table,keyData,keyLen);
    theTrans.theNdb = this;
    return theTrans;
  }

  public NdbTransactionImpl startTransaction(NdbTable table, String keyData) throws NdbApiException {
    NdbTransactionImpl theTrans  = realStartTransaction(table,keyData);
    theTrans.theNdb = this;
    return theTrans;
  }

  public NdbTransactionImpl startTransaction(NdbTable table) throws NdbApiException {
    NdbTransactionImpl theTrans  = realStartTransaction(table);
    theTrans.theNdb = this;
    return theTrans;
  }

  public NdbTransactionImpl startTransaction() throws NdbApiException {
    NdbTransactionImpl theTrans  = realStartTransaction();
    theTrans.theNdb = this;
    return theTrans;
  }
  public NdbTransactionImpl startTransaction(String aTableName, int keyData) throws NdbApiException {
    NdbTransactionImpl theTrans  = realStartTransaction(aTableName,keyData);
    theTrans.theNdb = this;
    return theTrans;
  }
  public NdbTransactionImpl startTransaction(String aTableName, String keyData) throws NdbApiException {
    NdbTransactionImpl theTrans  = realStartTransaction(aTableName,keyData);
    theTrans.theNdb = this;
    return theTrans;
  }

  %}
*/
%javamethodmodifiers Ndb::closeTransaction "protected";

%include "ndbapi/Ndb.i"
%include "ndbapi/NdbDictionary.i"

%typemap(javainterfaces) NdbTransaction "NdbTransaction";

/*
%javamethodmodifiers NdbTransaction::getNdbIndexOperation(Ndb*, const char*, const char*) "protected";
%javamethodmodifiers NdbTransaction::getNdbIndexScanOperation(Ndb*, const char*, const char*) "protected";
%rename(realGetNdbIndexOperation) getNdbIndexOperation(Ndb*, const char*, const char*);
%rename(realGetNdbIndexScanOperation) getNdbIndexScanOperation(Ndb*, const char*, const char*);

%typemap(javacode) NdbTransaction %{
  protected NdbImpl theNdb; 

  public NdbIndexOperation getNdbIndexOperation(String indexName, String tableName) throws NdbApiException { 
    return realGetNdbIndexOperation(theNdb,indexName,tableName);
  }  
  public NdbIndexScanOperation getNdbIndexScanOperation(String indexName, String tableName) throws NdbApiException { 
    return realGetNdbIndexScanOperation(theNdb,indexName,tableName);
  }  
  %}
*/

%rename(realClose) NdbTransaction::close;
%typemap(javacode) NdbTransaction %{
  public void close() { 
    this.realClose();
    this.swigCPtr=0;
  }
%}

%include "ndbapi/NdbTransaction.i"

%extend NdbTransaction { 
  
};

%typemap(javainterfaces) NdbOperation "NdbOperation,NdbInterpretedOperation";
%typemap(javabase) NdbOperation "NdbOperationResultsImpl";
%typemap(javacode) NdbOperation %{

public void equal(long columnId, BigInteger val) throws NdbApiException {
	equalUlong(columnId,val);
}

public void equal(long columnId, byte[] val) throws NdbApiException {
	equalBytes(columnId,val);
}

public void equal(long columnId, Calendar val) throws NdbApiException {
	equalDatetime(columnId,val);
}

public void equal(long columnId, int val) throws NdbApiException {
	equalInt(columnId,val);
}

public void equal(long columnId, long val) throws NdbApiException {
	equalLong(columnId,val);
}

public void equal(long columnId, String val) throws NdbApiException {
	equalString(columnId,val);
}

public void equal(long columnId, Timestamp val) throws NdbApiException {
	equalTimestamp(columnId,val);
}

public void equal(String columnName, BigInteger val) throws NdbApiException {
	equalUlong(columnName,val);
}

public void equal(String columnName, byte[] val) throws NdbApiException {
	equalBytes(columnName,val);
}

public void equal(String columnName, Calendar val) throws NdbApiException {
	equalDatetime(columnName,val);
}

public void equal(String columnName, int val) throws NdbApiException {
	equalInt(columnName,val);
}

public void equal(String columnName, long val) throws NdbApiException {
	equalLong(columnName,val);
}

public void equal(String columnName, String val) throws NdbApiException {
	equalString(columnName,val);
}

public void equal(String columnName, Timestamp val) throws NdbApiException {
	equalTimestamp(columnName,val);
}

 /**
   * getNdbBlobHandle is deprecated. Please use getBlobHandle instead
   * @param columnName
   * @return
   * @throws NdbApiException
   */
  @Deprecated
  public NdbBlob getNdbBlobHandle(String columnName) throws NdbApiException { 
    return getBlobHandle(columnName);
  }

 /**
   * getNdbBlobHandle is deprecated. Please use getBlobHandle instead
   * @param columnName
   * @return
   * @throws NdbApiException
   */
  @Deprecated
  public NdbBlob getNdbBlobHandle(long columnId) throws NdbApiException { 
    return getBlobHandle(columnId);
  }

  %}

%typemap(javaimports) NdbOperation %{
import java.sql.Timestamp;
import java.util.Calendar;
import java.math.BigInteger;
  %}



%javamethodmodifiers NdbOperation::getValue "protected";
%rename(realGetValue) getValue(const char* anAttrName);
%rename(realGetValue) getValue(Uint32 anAttrId);
%rename(realGetValue) getValue(const NdbDictColumn* col);

%typemap(jstype) NdbOperation::LockMode "NdbOperation.LockMode"
%typemap(javaout) NdbOperation::LockMode { 
        return NdbOperation.LockMode.swigToEnum(ndbjJNI.NdbOperationImpl_getLockMode(swigCPtr, this));
}


%include "ndbapi/NdbOperation.i"

%typemap(jstype) NdbScanOperation::ScanFlag "NdbScanOperation.ScanFlag"
%typemap(javaout) NdbScanOperation::ScanFlag { 
        return NdbScanOperation.ScanFlag.swigToEnum(ndbjJNI.NdbScanOperationImpl_getScanFlag(swigCPtr, this));
}

%typemap(javainterfaces) NdbScanOperation "NdbScanOperation";

%javamethodmodifiers NdbScanOperation::getNdbScanFilter "protected";
%rename(realGetNdbScanFilter) getNdbScanFilter();

%typemap(javaimports) NdbScanOperation "import java.util.ArrayList;"
%typemap(javacode) NdbScanOperation %{
  
  public NdbScanFilter getNdbScanFilter() throws NdbApiException {
    return new NdbScanFilterImpl(this);
  }

  public void readTuples(NdbOperation.LockMode mode, NdbScanOperation.ScanFlag scanFlag, long parallel,
			long batch) throws NdbApiException { 
    readTuples(mode,scanFlag.type,parallel,batch);
  }

  public void readTuples(NdbOperation.LockMode mode, ArrayList<NdbScanOperation.ScanFlag> scanFlag, long parallel,
			long batch) throws NdbApiException {
    long flagVal = 0;
    for (NdbScanOperation.ScanFlag s : scanFlag) { 
      flagVal = flagVal | s.type;
    }
    readTuples(mode,flagVal,parallel,batch);
  }
  

  %}

%include "ndbapi/NdbScanOperation.i"

%typemap(javainterfaces) NdbIndexOperation "NdbIndexOperation";

%include "ndbapi/NdbIndexOperation.i"

%typemap(jstype) NdbIndexScanOperation::BoundType "NdbIndexScanOperation.BoundType"
%typemap(javaout) NdbIndexScanOperation::BoundType { 
        return NdbIndexScanOperation.BoundType.swigToEnum(ndbjJNI.NdbIndexScanOperationImpl_getBoundType(swigCPtr, this));
}
%typemap(javainterfaces) NdbIndexScanOperation "NdbIndexScanOperation";

%include "ndbapi/NdbIndexScanOperation.i"

%javamethodmodifiers NdbEventOperation::getValue "protected";
%javamethodmodifiers NdbEventOperation::getPreValue "protected";
%javamethodmodifiers NdbEventOperation::getPreBlobHandle "protected";
%rename(realGetPreValue) getPreValue(const char* anAttrName);
%rename(realPreBlobHandle) getBlobHandle(const char* anAttrName);
%typemap(javainterfaces) NdbEventOperation "NdbEventOperation";
%typemap(javabase) NdbEventOperation "NdbEventOperationResultsImpl";
%typemap(jstype) NdbEventOperation::State "NdbEventOperation.State"
%typemap(javaout) NdbEventOperation::State { 
        return NdbEventOperation.State.swigToEnum(ndbjJNI.NdbEventOperationImpl_getState(swigCPtr, this));
}


%include "ndbapi/NdbEventOperation.i"

%typemap(javainterfaces) NdbRecAttr "NdbRecAttr"; 

%include "ndbapi/NdbRecAttr.i"

%typemap(javacode)  NdbBlob %{

  private byte[] theBlob;	
  
  public void getValue(int length) { 
	  theBlob=new byte[length];
  }

  public byte[] getData() throws NdbApiException { 
	  this.readData(theBlob, theBlob.length);
	  return theBlob;
  }
  %}

%typemap(javainterfaces) NdbBlob "NdbBlob";

%typemap(jstype) NdbBlob::State "NdbBlob.State"
%typemap(javaout) NdbBlob::State { 
        return NdbBlob.State.swigToEnum(ndbjJNI.NdbBlobImpl_getState(swigCPtr, this));
}

%include "ndbapi/NdbBlob.i"
%include "NdbErrorDoc.i"

%typemap(javainterfaces) NdbScanFilter "NdbScanFilter";
%typemap(jstype) NdbScanFilter::Group "NdbScanFilter.Group"
%typemap(javaout) NdbScanFilter::Group { 
        return NdbScanFilter.Group.swigToEnum(ndbjJNI.NdbScanFilterImpl_getGroup(swigCPtr, this));
}
%typemap(jstype) NdbScanFilter::BinaryCondition "NdbScanFilter.BinaryCondition"
%typemap(javaout) NdbScanFilter::BinaryCondition { 
        return NdbScanFilter.BinaryCondition.swigToEnum(ndbjJNI.NdbScanFilterImpl_getBinaryCondition(swigCPtr, this));
}

%include "ndbapi/NdbError.i"
%include "ndbapi/NdbScanFilter.i"

