package testsuite;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.Properties;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbClusterConnection;
import com.mysql.cluster.ndbj.NdbFactory;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.ndbj;


public class BaseNdbjTestCase extends testsuite.BaseTestCase {

	protected static final String CONFIG_FILENAME="ndbj.props";
	protected static String MYSQLD_HOST="";
	protected static String NDB_MGMD_CONNECTSTRING="";
	protected static String DB_NAME="test";
	protected static String TABLE_NAME="";
	protected static String LIB_PATH = "";
	protected static String USERNAME  = "root";
	protected static String PASSWORD = "";
	protected static String CLUSTER_LOG="";
	protected static Connection MYSQL_CONN=null;
	
	protected static NdbClusterConnection conn=null;
	protected com.mysql.cluster.ndbj.Ndb ndb=null;
	protected NdbTransaction trans=null;
	
	static { 
  	    System.loadLibrary("ndbj");
    	ndbj.ndb_init();
	}

	public BaseNdbjTestCase(String arg0) {

		super(arg0);
		/**
		 * Set environment variables
		 */

		File f =null;
		try {
			InputStream in=null;
			f = new File(CONFIG_FILENAME);
			if (f.exists()){
				if (f.canRead()){
					in = new FileInputStream(f);
					}
				else {
					System.out.println("Could not open " + CONFIG_FILENAME);
					assertFalse(true);
					}
				}
			else {
				System.out.println("Could not find " + CONFIG_FILENAME);
				assertFalse(true);
				}

		    Properties p = new Properties(System.getProperties());
		    p.load(in);

			// set the system properties
	        System.setProperties(p);

	        BaseNdbjTestCase.MYSQLD_HOST = System.getProperty("ndbj.mysqld");
			if (BaseNdbjTestCase.MYSQLD_HOST == null) {
				System.out.println("ndbj.mysql not defined in configuration file: + " + CONFIG_FILENAME);
				assertFalse(true);
			}

			BaseNdbjTestCase.NDB_MGMD_CONNECTSTRING= System.getProperty("ndbj.connectstring");
			if (BaseNdbjTestCase.NDB_MGMD_CONNECTSTRING == null) {
				System.out.println("ndbj.connectstring not defined in configuration file: + " + CONFIG_FILENAME);
				assertFalse(true);
			}

			BaseNdbjTestCase.DB_NAME= System.getProperty("ndbj.testDatabase");
			if (BaseNdbjTestCase.DB_NAME == null) {
				System.out.println("ndbj.testDatabase was empty. Setting it to 'test' db.");
			}

			BaseNdbjTestCase.LIB_PATH= System.getProperty("ndbj.libpath");
			if (BaseNdbjTestCase.LIB_PATH == null) {
			}

			BaseNdbjTestCase.USERNAME = System.getProperty("ndbj.username");
			if (BaseNdbjTestCase.USERNAME == null) {
				System.out.println("ndbj.username was empty. Setting it to 'root'.");
			}

			BaseNdbjTestCase.PASSWORD = System.getProperty("ndbj.password");
			if (BaseNdbjTestCase.PASSWORD == null) {
				System.out.println("ndbj.password was empty. Setting it to empty string ''");
			}

			BaseNdbjTestCase.CLUSTER_LOG= System.getProperty("ndbj.clusterLog");
			if (BaseNdbjTestCase.CLUSTER_LOG== null) {
			}
		}
		catch (java.io.FileNotFoundException e) {
			System.err.println(e.getMessage());
			assertFalse(true);
		}
		catch (java.io.IOException e) {
			System.err.println(e.getMessage());
			assertFalse(true);
			}
		catch (NullPointerException e) {
			System.err.println(e.getMessage());
			assertFalse(true);
			}
		catch (IllegalArgumentException e) {
			System.err.println(e.getMessage());
			assertFalse(true);
		}
		catch (SecurityException e) {
			System.err.println(e.getMessage());
			assertFalse(true);
		}


/*
		Class clazz = ClassLoader.class;
        Field field;
        boolean accessible;
        try {
        // Reset the "sys_paths" field of the ClassLoader to null.
        field = clazz.getDeclaredField("sys_paths");
        accessible = field.isAccessible();
        if (!accessible)
            field.setAccessible(true);
        // Reset it to null so that whenever "System.loadLibrary" is called, it will be reconstructed with the changed value.
        field.set(clazz, null);
        System.setProperty("java.library.path", LIB_PATH);
        }
        catch (java.lang.UnsatisfiedLinkError e) {
            System.err.println(e.getMessage());
        }
        catch (NoSuchFieldException e) {
            System.err.println(e.getMessage());
        }
        catch (IllegalAccessException e) {
            System.err.println(e.getMessage());
        }
*/


	}

	public void setUp() throws Exception {
	      /**
	       *  Connect to cluster, done once for each Application
	       */
		super.setUp();
    	try { 
    		System.out.print("Connecting to cluster...");
      	  	conn = NdbFactory.createNdbClusterConnection(NDB_MGMD_CONNECTSTRING);

  		    conn.connect(5,3,true);
  		    conn.waitUntilReady(30,0);
		      /**
		       *  Get a connection to NDB (once / thread)
		       */
		    ndb = conn.createNdb("test",1024);

  		    System.out.println("connected");
    	} catch (NdbApiException e) { 
      		throw new Exception("Failed initialization");
      	}
	}

	public void tearDown() throws Exception {


		System.out.println("Closing transaction");
		if (trans != null)
			trans.close();

		ndb.close();
		System.out.println("Closing connection");
		if (conn != null)
			conn.close();
		super.tearDown();
	}

}
