package testsuite.ndbj;

import java.util.*;

import testsuite.BaseNdbjTestCase;
import com.mysql.cluster.ndbj.*;

public class ImmediatePollTimeoutTest extends BaseNdbjTestCase {

	static boolean alreadySetUp                   = false;
	static long    ourTotalTransactionCount       = 0;
	static long    ourOutstandingTransactionCount = 0;
	static long    ourCallbackCount               = 0;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// GENERAL STUFF
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	static int MAXIMUM_OPERATION_VOLUME       = 300; // TO DO: Calculate this

	static int NUMBER_OF_ELEMENT_INSERTIONS   = 50;

	static int NUM_REQUEST_TRANSACTIONS       = 2;
	static int REQUEST_TRANSACTIONS_TIMEOUT   = 0;

	static int FORCE_SEND                     = 1;

	static boolean USE_SYNCHRONOUS            = false;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TABLE METADATA
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	static String tablename      = "t_immediatepolltimeouttest";

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// INITIALISATION CODE
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////


	public ImmediatePollTimeoutTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();

		if (!alreadySetUp) {
			// Create required tables

			System.out.print("Creating tables...");

			this.createTable(tablename, 
				"    (ATTR1 INT UNSIGNED auto_increment not null," +
				"     ATTR2 INT UNSIGNED NOT NULL," +
				"     PRIMARY KEY (ATTR1) )" + 
				"  ENGINE=NDBCLUSTER");

			System.out.println("done.");

			System.out.print("Storing column indices...");
			System.out.println("done.");

			alreadySetUp = true;
		}
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TEST CODE
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public void testOperation() {
		
		HashMap<Integer,BaseCallback> hash=new HashMap<Integer,BaseCallback>();
	
		try {
			for (int loop = 0;
			loop < 100;
			loop++) {
				// Process request transaction

				NdbOperation orderOp;
				BaseCallback callback;

				for (int elementCount = 0;
				     elementCount < NUMBER_OF_ELEMENT_INSERTIONS;
				     elementCount++) {
					int id = (NUMBER_OF_ELEMENT_INSERTIONS * loop) + elementCount;

					trans = ndb.startTransaction();
					orderOp = trans.getNdbOperation(tablename);
					orderOp.insertTuple();
					orderOp.equalInt("ATTR1", id);
					orderOp.setInt("ATTR2", id);

					callback = new MyCallback(hash, id, ndb, trans, orderOp.resultData());
					hash.put(callback.hashCode(), callback);

					trans.executeAsynchPrepare(ExecType.Commit,
							callback,
							AbortOption.AbortOnError);

					ourOutstandingTransactionCount++;
				}

				ourCallbackCount = 0;

				ndb.sendPreparedTransactions(FORCE_SEND);

				long then = System.nanoTime();
				ndb.pollNdb(REQUEST_TRANSACTIONS_TIMEOUT);
				long now = System.nanoTime();

				ourOutstandingTransactionCount = ourOutstandingTransactionCount - ourCallbackCount;
				System.out.println("Poll took " + (now - then) + " nanos (" + ourCallbackCount + 
							" txs received) [" + ourOutstandingTransactionCount + " outstanding]");
			}

			System.out.println("Performing cooldown");

			ourCallbackCount = 0;

			long then = System.nanoTime();
			ndb.pollNdb(100000,100000);
			long now = System.nanoTime();

			ourOutstandingTransactionCount = ourOutstandingTransactionCount - ourCallbackCount;
			System.out.println("Closedown poll took " + (now - then) + " nanos (" + ourCallbackCount + 
						" txs received) [" + ourOutstandingTransactionCount + " outstanding]");
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			e.printStackTrace();
			assertFalse(true);
		}
		finally {
			// Do something ;o)
		}
	}

	class HashCallback extends BaseCallback {
	

	    int                            myCallbackNum;
	    Ndb                            myNdb;
	    NdbResultSet                   myResults;
	    HashMap<Integer, BaseCallback> myMap;

	    public HashCallback(HashMap<Integer, BaseCallback> theMap, int theCallbackNum, Ndb theNdb, NdbTransaction theTrans, NdbResultSet theResults) {
	    	super(theTrans);
	    	myMap         = theMap;
	    	myCallbackNum = theCallbackNum;
	    	myNdb         = theNdb;
	    	myResults     = theResults;
	    }
    
	    public void callback(int res) { 

		if (res!=0) System.out.println("HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHASSSSSSSSSSSSSSSSSSSSHHH");
	
		theTrans.close();
		myMap.remove(this.hashCode());

		ourCallbackCount++;
	    }
    
	}

	class MyCallback extends HashCallback {

	    public MyCallback(HashMap<Integer, BaseCallback> theMap,int theCallbackNum, Ndb theNdb, NdbTransaction theTrans, NdbResultSet theResults) {
		super(theMap, theCallbackNum, theNdb, theTrans, theResults);
	    }

	}

}
