package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbApiTemporaryException;
import com.mysql.cluster.ndbj.NdbApiUserErrorPermanentException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbTransaction;

import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class InsertRetryTest extends BaseNdbjTestCase {

	protected static final String tablename = "insert_retry_test";
	
	protected static final String col1 = "id";

	protected static final String col2 = "name";

	protected static final String col2Val = "montyµ";

	protected static final int MAX_RETRIES = 3;

	protected static final int NUM_INSERTS = 2;

	public static void main(String[] args) {
		junit.textui.TestRunner.run(InsertRetryTest.class);
	}

	public InsertRetryTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception { 
		super.setUp();

		System.out.println("Creating table: " + tablename);
		this.createTable(tablename, "(id int, name varchar(56), primary key(id)) engine=ndbcluster default charset=utf8");
		
	}

	/*
	 * Test method for 'com.mysql.ndbapi.Ndb.destroy()'
	 */
	public void testInsert() throws java.sql.SQLException {

		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			try {
				trans = ndb.startTransaction(tablename,tablename);
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getNdbOperation(tablename);
					op.insertTuple();
					op.equal(col1, i);
					String setting = col2Val + Integer.toString(i);
					op.setString(col2, setting);
				}

				trans.execute(ExecType.Commit,AbortOption.AbortOnError, true);

				done = true;
				assertTrue(true);
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				System.out
						.println("Temporary error. Retrying insert. Num Retries == "
								+ numRetries + " error msg:" + e.getMessage());
			} catch (NdbApiException e) {
				System.out.println("Caught NdbApiException in Insert:\t"
						+ e.getMessage());
				assertTrue(false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}
		for (int i = 0; i < NUM_INSERTS; i++) { 
			String nameVal = (String)this.getSingleValue(tablename, "name", "where id=" + Integer.toString(i));
			assertEquals(nameVal,col2Val + Integer.toString(i));
		}
	}
	
	public void testRead() { 
		
		// TODO: Replace this with JDBC inserting of data
		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			try {
				trans = ndb.startTransaction(tablename,tablename);
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getNdbOperation(tablename);
					op.insertTuple();
					op.equal(col1, i);
					String setting = col2Val + Integer.toString(i);
					op.setString(col2, setting);
				}

				trans.execute(ExecType.Commit,AbortOption.AbortOnError, true);

				done = true;
				assertTrue(true);
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				System.out.println("Temporary error. Retrying insert. Num Retries == "
								+ numRetries + " error msg:" + e.getMessage());
			} catch (NdbApiException e) {
				System.out.println("Caught NdbApiException in Insert:\t"
						+ e.getMessage());
				assertTrue(false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}

		try {
			System.out.println("Reading the inserted rows");
			trans = ndb.startTransaction();

			NdbResultSet[] rs = new NdbResultSet[NUM_INSERTS];
			for (int i = 0; i < NUM_INSERTS; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);
				op.readTuple(LockMode.LM_Read);
				op.equal(col1, i);
				op.getValue(col2);
				rs[i] = op.resultData();
			}

			trans.execute(ExecType.Commit,AbortOption.AbortOnError, true);

			int j=0;
			for (NdbResultSet r : rs) {
				while (r.next()) {
					String val = r.getString(col2);
					System.out.println("Read: " + val + " Expected: " + col2Val+Integer.toString(j));
					assertEquals(val,col2Val+Integer.toString(j));
				}
				j++;
			}

			System.out.println("Finished reading");
			assertTrue(true);
		} catch (NdbApiUserErrorPermanentException e) {
			System.out.println("User Permanent error. Retrying insert. Num Retries == "
							+ e.getMessage());
		} catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t"
					+ e.getMessage());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}
	
	}

	public void testDelete() throws java.sql.SQLException {
		
		// TODO: Repleace with JDBC data insert
		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			try {
				trans = ndb.startTransaction(tablename,tablename);
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getNdbOperation(tablename);
					op.insertTuple();
					op.equal(col1, i);
					String setting = col2Val + Integer.toString(i);
					op.setString(col2, setting);
				}

				trans.execute(ExecType.Commit,AbortOption.AbortOnError, true);

				done = true;
				assertTrue(true);
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				System.out
						.println("Temporary error. Retrying insert. Num Retries == "
								+ numRetries + " error msg:" + e.getMessage());
			} catch (NdbApiException e) {
				System.out.println("Caught NdbApiException in Insert:\t"
						+ e.getMessage());
				assertTrue(false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}
		try {
			System.out.println("Deleting the inserted rows");
			trans = ndb.startTransaction();

			for (int i = 0; i < NUM_INSERTS; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);
				op.deleteTuple();
				op.equal(col1, i);
			}

			trans.execute(ExecType.Commit,AbortOption.AbortOnError, true);

			System.out.println("Finished delete");
			assertTrue(true);
		} catch (NdbApiUserErrorPermanentException e) {
			System.out
					.println("User Permanent error. Retrying insert. Num Retries == "
							+ e.getMessage());
		} catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t"
					+ e.getMessage());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}
		int rowcount = this.getRowCount(tablename);
		assertEquals(rowcount,0);

	}

}
