package testsuite.ndbj;

import java.util.Random;

import testsuite.BaseNdbjTestCase;

import junit.framework.AssertionFailedError;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbApiTemporaryException;
import com.mysql.cluster.ndbj.NdbApiUserErrorPermanentException;
import com.mysql.cluster.ndbj.NdbError;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class Latin1InsertReadTest extends BaseNdbjTestCase {

	protected static final String tablename = "t_latin1";
	
	protected static final String col1 = "id";

	protected static final String col2 = "name";

	protected static final String col2Val = "jimµ";

	protected static final int MAX_RETRIES = 3;

	protected static final int NUM_INSERTS = 2;

	public static void main(String[] args) {
		junit.textui.TestRunner.run(Latin1InsertReadTest.class);
	}

	public Latin1InsertReadTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception { 
		super.setUp();

		System.out.println("Creating table: " + tablename);
		this.createTable(tablename,	"(`id` VARCHAR(100) PRIMARY KEY, `name` VARCHAR(256)) ENGINE=ndb default charset=latin1");
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Test method for 'com.mysql.ndbapi.Ndb.destroy()'
	 */
	public void testLatin1() {

		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			try {
				trans = ndb.startTransaction();
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans.getNdbOperation(tablename);
					op.insertTuple();
					if (i != 0)
						op.equal(col1, Integer.toString(i));
					else
						op.equal(col1,col2Val);
					op.setString(col2, col2Val);
					System.out.println("Inserted:" + col2Val);
				}

				trans.execute(ExecType.Commit,
								AbortOption.AbortOnError, true);

				done = true;
				assertTrue(true);
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				System.out
						.println("Temporary error. Retrying insert. Num Retries == "
								+ numRetries + " error msg:" + e.getMessage());
				System.out.println(e.getMessage());
			} catch (NdbApiException e) {
				System.out.println("Caught NdbApiException in Insert:\t"
						+ e.getMessage());

				System.out.println(e.getErrorObj().getMessage());
				assertTrue(false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
	}

		try {
			System.out.println("Reading the inserted rows");
			trans = ndb.startTransaction();

			NdbResultSet[] rs = new NdbResultSet[NUM_INSERTS];
			for (int i = 0; i < NUM_INSERTS; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);
				op.readTuple(LockMode.LM_Read);
				if (i != 0)
					op.equal(col1, Integer.toString(i));
				else
					op.equal(col1,col2Val);

				op.getValue(col2);
				rs[i] = op.resultData();
			}

			trans.execute(ExecType.Commit,
							AbortOption.AbortOnError, true);

			for (NdbResultSet r : rs) {
				while (r.next()) {
					String val = r.getString(col2);
					System.out.println("Read: " + val);
					if (val.compareTo(col2Val)!=0)
						assertFalse(true);
				}
			}

			System.out.println("Finished reading");
			assertTrue(true);
		} catch (NdbApiUserErrorPermanentException e) {
			System.out.println("User Permanent error. Retrying insert. Num Retries == "
							+ e.getMessage());
		} catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t"
					+ e.getMessage());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}


		try {
			System.out.println("Deleting the inserted rows");
			trans = ndb.startTransaction();

			for (int i = 0; i < NUM_INSERTS; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);
				op.deleteTuple();
				if (i != 0)
					op.equal(col1, Integer.toString(i));
				else
					op.equal(col1,col2Val);

			}

			trans.execute(ExecType.Commit,
							AbortOption.AbortOnError, true);

			System.out.println("Finished delete");
			assertTrue(true);
		} catch (NdbApiUserErrorPermanentException e) {
			System.out
					.println("User Permanent error. Retrying insert. Num Retries == "
							+ e.getMessage());
		} catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t"
					+ e.getMessage());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

	}

}
