package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbError;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbScanFilter;
import com.mysql.cluster.ndbj.NdbScanOperation;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.NdbDictionary;
import com.mysql.cluster.ndbj.NdbTable; 

/**
 *
create table t_multipart_primary (a int, b int, c varbinary(20), PRIMARY KEY(a,b)) engine=ndb;
insert into t_multipart_primary values(1,11,"jim.......");

 * @author jdowling
 *
 */
public class MultiPartPrimaryKeyStringTest extends BaseNdbjTestCase {

	protected static final String tablename = "t_multipart_primary";
	private static final String col1Val="jd";
	private static final String col2Val="jp";
	private static final String col3Val="jim";

	private static final int NUM_ITERATIONS=500;
	protected String col1="a";
	protected String col2="b";
	protected String col3="c";

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MultiPartPrimaryKeyStringTest.class);
	}

	public MultiPartPrimaryKeyStringTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
				"(`" + col1 + "` VARCHAR(256) NOT NULL, `"
				+ col2 + "` VARCHAR(256) NOT NULL, `"
				+ col3 + "` VARBINARY(256), PRIMARY KEY(a,b)) ENGINE=ndb"
				+ " PARTITION BY KEY(a,b);");
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}


	public void testMultiPartPrimaryKey() {


		System.out.println("Starting a transaction for table " + tablename);

	    for (int i=0; i < NUM_ITERATIONS; i++) {

		try {
			String pk1, pk2;

			if (i > 0) {
				pk1 = col1Val + i;
				pk2 = col2Val + i;
			}
			else {
				pk1 = col1Val;
				pk2 = col2Val;
			}


		    trans = ndb.startTransaction(tablename,col1Val+col2Val);
	    	NdbOperation op = trans.getNdbOperation(tablename);

				op.insertTuple();

				op.equal(col1,pk1);
				op.equal(col2,pk2);

				String col3Bytes = col3Val+Integer.toString(i);
				op.setBytes(col3,col3Bytes.getBytes()); //

			    trans.execute(ExecType.Commit,
					      AbortOption.AbortOnError,
					      true);
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}
    }
	    System.out.println("Inserted " + NUM_ITERATIONS + " rows");

		System.out.println("Starting transactions using multi-part PRIMARY KEY distribution hint");

	   for (int i=0; i < NUM_ITERATIONS; i++) {

		try {

			// distribution hint is value of both columns in primary key
			// TODO: Do we need to actually support a (TableName,Val1,Val2) form here?
			trans = ndb.startTransaction(tablename,col1Val+col2Val);

			NdbOperation op = trans.getNdbOperation(tablename);

		    op.readTuple(LockMode.LM_Exclusive);

			if (i > 0) {
				String pk1 = col1Val + i;
				op.equal(col1,pk1);
				String pk2 = col2Val + i;
				op.equal(col2,pk2);
			}
			else {
				op.equal(col1,col1Val);
				op.equal(col2,col2Val);
			}
			op.getValue(col3);

		    NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

		    byte[] c=null;
		    int numRes=0;
			while(rs.next())
			{
				numRes++;
			    c = rs.getBytes(col3);
			}
			assertEquals(numRes,1);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		}
	   assertTrue(true);

		try {
			System.out.println("Starting a scan operation");
		    trans = ndb.startTransaction();

		    NdbDictionary theDict = ndb.getDictionary();
		    NdbTable theTable = theDict.getTable(tablename);
			NdbScanOperation op =
				trans.getNdbScanOperation(theTable);

		    op.readTuples(LockMode.LM_CommittedRead,NdbScanOperation.ScanFlag.NO_FLAG,0,0);

		    op.getValue(col1);
		    op.getValue(col2);
			op.getValue(col3);

			
			NdbScanFilter sf = op.getNdbScanFilter();
			 
			sf.begin();
			// TODO: Fix naming issue
			sf.cmpString(NdbScanFilter.BinaryCondition.COND_EQ, col1, col1Val);
			sf.end();
			
			NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

		    int numRes = 0;
			while(rs.next())
			{
			
					numRes++;
					String c1 = rs.getString(col1);
					if (rs.wasNull() == true)
						assertFalse(true);
					String c2 = rs.getString(col1);
					if (rs.wasNull() == true)
						assertFalse(true);
					byte[] c = rs.getBytes(col3);
					if (rs.wasNull() == true)
						assertFalse(true);
				
			}
			assertEquals(numRes, 1);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting a setbound-eq index scan");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getNdbIndexScanOperation("PRIMARY",tablename);

            op.readTuples(LockMode.LM_Exclusive,NdbScanOperation.ScanFlag.NO_FLAG,0,0);

			op.equal(col1,col1Val);
			op.equal(col2,col2Val);
			op.getValue(col3);

		    NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

		    byte[] c =null;
		    int numRes=0;
			while(rs.next())
			{
				numRes++;
			    c = rs.getBytes(col3);
			}
			assertEquals(numRes,1);

		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting a setbound-gt index scan");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getNdbIndexScanOperation("PRIMARY",tablename);

			op.readTuples(LockMode.LM_Exclusive, NdbScanOperation.ScanFlag.NO_FLAG,0,0);

			op.whereGreaterThanEqualTo(col1,col1Val);
			op.whereGreaterThan(col2,col2Val);
			op.getValue(col3);

		    NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

		    int numRes=0;
			while(rs.next())
			{
			    byte[] c = rs.getBytes(col3);
			    numRes++;
			}
			assertEquals(numRes,NUM_ITERATIONS-1);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

	
		try {
		    trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);

			op.updateTuple();
			op.equal(col1,col1Val);
			op.equal(col2,col2Val);
			op.setBytes(col3, (col3Val + "_jd").getBytes());

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished update");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in update:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}


		try {
			System.out.println("Starting an delete transaction");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbOperation op = trans.getNdbOperation(tablename);

			op.deleteTuple();

			op.equal(col1,col1Val);
			op.equal(col2,col2Val);

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished delete");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

	}

}
