package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;
import testsuite.TestCallback;
import com.mysql.cluster.ndbj.*;

public class MultipleAsyncTest extends BaseNdbjTestCase {

	static boolean alreadySetUp = false;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// GENERAL STUFF
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	static int MAXIMUM_OPERATION_VOLUME       = 300; // TO DO: Calculate this

	static int NUMBER_OF_ELEMENT_INSERTIONS   = 25;

	static int NUM_REQUEST_TRANSACTIONS       = 2;
	static int REQUEST_TRANSACTIONS_TIMEOUT   = 1000;

	static int FORCE_SEND                     = 1;

	static boolean USE_SYNCHRONOUS            = false;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TABLE METADATA
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	static String tablename1      = "t_multi_async1";
	static String tablename2      = "t_multi_async2";
	static String tablename3      = "t_multi_async3";



	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// INITIALISATION CODE
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////


	public MultipleAsyncTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();

		if (!alreadySetUp) {
			// Create required tables

			System.out.print("Creating tables...");

			this.createTable(tablename1, 
				"    (ATTR1 INT UNSIGNED auto_increment not null," +
				"     ATTR2 INT UNSIGNED NOT NULL," +
				"     PRIMARY KEY (ATTR1) )" + 
				"  ENGINE=NDBCLUSTER");
			this.createTable(tablename2, 
					"    (ATTR1 INT UNSIGNED auto_increment not null," +
					"     ATTR2 INT UNSIGNED NOT NULL," +
					"     PRIMARY KEY (ATTR1) )" + 
					"  ENGINE=NDBCLUSTER");			
			this.createTable(tablename3, 
							"    (ATTR1 INT UNSIGNED auto_increment not null," +
							"     ATTR2 INT UNSIGNED NOT NULL," +
							"     PRIMARY KEY (ATTR1) )" + 
							"  ENGINE=NDBCLUSTER");

			System.out.println("done.");
			System.out.print("Storing column indices...");

	

			alreadySetUp = true;
		}
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TEST CODE
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public void testMultipleOperation() {

		try {
			for (int loop = 0;
			loop < 100;
			loop++) {
				// Process request transaction


				BaseCallback callback;

				// Process response transaction

				trans = ndb.startTransaction();

				NdbOperation firstOp;
				NdbOperation secondOp;
				NdbOperation thirdOp;

				for (int elementCount = 0;
				elementCount < NUMBER_OF_ELEMENT_INSERTIONS;
				elementCount++) {
					int id = (NUMBER_OF_ELEMENT_INSERTIONS * loop) + elementCount;

					trans = ndb.startTransaction();

					firstOp = trans.getNdbOperation(tablename1);

					firstOp.insertTuple();

					firstOp.equal("ATTR1", id);
					firstOp.setInt("ATTR2", id);

					if (!USE_SYNCHRONOUS) {
						callback = new TestCallback(tablename1, id, ndb, trans, firstOp.resultData());

						trans.executeAsynchPrepare(ExecType.Commit,
								callback,
								AbortOption.AbortOnError);
					}

					trans = ndb.startTransaction();

					secondOp = trans.getNdbOperation(tablename1);

					secondOp.insertTuple();

					secondOp.equal("ATTR1", id);
					secondOp.setInt("ATTR2", id);

					if (!USE_SYNCHRONOUS) {
						callback = new TestCallback(tablename2, id, ndb, trans, firstOp.resultData());

						trans.executeAsynchPrepare(ExecType.Commit,
								callback,
								AbortOption.AbortOnError);
					}

					trans = ndb.startTransaction();

					thirdOp = trans.getNdbOperation(tablename3);


					thirdOp.insertTuple();

					thirdOp.equal("ATTR1", id);
					thirdOp.setInt("ATTR2", id);

					if (!USE_SYNCHRONOUS) {
						callback = new TestCallback(tablename3, id, ndb, trans, firstOp.resultData());

						trans.executeAsynchPrepare(ExecType.Commit,
								callback,
								AbortOption.AbortOnError);
					}
				}

				if (USE_SYNCHRONOUS) {
					trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);
				}
				else {

					ndb.sendPollNdb(REQUEST_TRANSACTIONS_TIMEOUT, NUM_REQUEST_TRANSACTIONS, FORCE_SEND);	
				}
			}

			System.out.println("Performing cooldown");

			ndb.sendPollNdb(100000, 100000, FORCE_SEND);	

		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			e.printStackTrace();
			junit.framework.Assert.assertTrue(true);
			assertFalse(true);
		}
		finally {
			if (trans!=null) {
				trans.close();
			}
		}
	}



}
