package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbBlob;
import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbClusterConnectionPermanentException;
import com.mysql.cluster.ndbj.NdbClusterConnectionTemporaryException;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

/**
 * With the mysql client, do the following:
 * 1. create a table 't1' in the database 'test'.
 * 2. insert some values into the table
 *
 CREATE TABLE `t1` (
 `a` int(11) NOT NULL,
 `b` blob,
 PRIMARY KEY (`a`)
 ) ENGINE=ndbcluster DEFAULT CHARSET=latin1 PARTITION BY KEY ();
 INSERT INTO t1 VALUES (1,'jim.......'), (2, 'rob...'), (3, NULL), (4, 'frazer...xxx.....');
 *
 */

public class NdbBlobTest extends BaseNdbjTestCase {

	private static final String tablename = "t_blob";

	private static final String col1 = "a"; //int

	private static final String col2 = "b"; //blob

	private static String bData = "";

	private static final int BLOB_SIZE = 9000;

	private static final int NUM_ROWS=3;

	public NdbBlobTest(String arg0) {
		super(arg0);
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbBlobTest.class);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
				"(`" + col1 + "` INT PRIMARY KEY, `"
				+ col2 + "` BLOB) ENGINE=ndb;");
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Test method for 'com.mysql.cluster.ndbj.NdbOperation.getNdbBlobHandle(int)'
	 */
	public final void testBlob() {
	

		try {

			bData="ROB...";
			for (int i = 0; i < NUM_ROWS; i++) {
				trans = ndb.startTransaction();
				NdbOperation op = trans.getNdbOperation(tablename);
				op.insertTuple();
				op.equal(col1, i);
				NdbBlob bh = op.getNdbBlobHandle(col2);
				bData = bData.concat(bData);
				if (i == 0) {
					bh.setNull();
				}
				else {
					bh.setValue(bData.getBytes());
				}


				trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

				trans.close();
			}

			assertTrue(true);
		} catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("Starting a transaction for table " + tablename);

		NdbBlob blob = null;
		try {
			long t1 = System.currentTimeMillis();
			trans = ndb.startTransaction();

			NdbIndexScanOperation op = trans.getNdbIndexScanOperation(
					"PRIMARY", tablename);

			op.readTuples(NdbOperation.LockMode.LM_Exclusive, 0, 0);
			op.setBoundInt(col1, NdbIndexScanOperation.BoundType.BoundLE, 0);
			op.getBlob(col2, 8000); // <--- Knowledge of Blob Length
			op.getValue(col1);

			NdbResultSet rs = op.resultData();

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			while (rs.next()) {
				int id = rs.getInt(col1);
				blob = rs.getBlob(col2);
				if (id == 0) {
					assertEquals(blob.getNull(), true);
					System.out.println("First entry was null");
					continue;
				}
				System.out.println("\nid=" + id);

				if (!blob.getNull()) {
					byte[] b = blob.getData();
					// TODO: Fix BLOB getLength()
					int len = blob.getLength().intValue();
					System.out.println("Length=" + len);
					byte[] input = bData.getBytes();
					for (int i = 0; i < len; i++) {
						System.out.print((char) b[i]);
						assertEquals(b[i],input[i]);
					}
					System.out.println();
				} else {
					System.out.println("NULL blob");
				}
			}

			long t2 = System.currentTimeMillis();
			long tot = t2 - t1;
			System.out.println("--END: " + tot);

			assertTrue(true);
		} 
		catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}
		NdbBlob b = null;
		try {
			long t1 = System.currentTimeMillis();
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);

			op.readTuple(NdbOperation.LockMode.LM_Read);
			op.equal(col1, 1);

			op.getBlob(col2);

			NdbResultSet rs = op.resultData();
			b = rs.getBlob(col2);

			trans.execute(ExecType.NoCommit,
					AbortOption.AbortOnError, true);

			System.out.println("length of Blob : " + b.getLength());

			// TODO: Fix BLOB getLength()
			int len = b.getLength().intValue();
			byte[] buff = new byte[len];
			b.readData(buff, len);

			System.out.println("START---");
			for (int i = 0; i < len; i++)
				System.out.print((char) buff[i]);
			System.out.println("--END");

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			long t2 = System.currentTimeMillis();
			long tot = t2 - t1;
			System.out.println("--END: " + tot);

			assertTrue(true);
		} 
		catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

		blob = null;
		try {
			long t1 = System.currentTimeMillis();
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);

			op.readTuple(NdbOperation.LockMode.LM_Read);
			op.equal(col1, 1);
			op.getBlob(col2, 8000); // <--- Knowledge of Blob Length

			NdbResultSet rs = op.resultData();
			blob = rs.getBlob(col2);

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			if (!blob.getNull()) {
				byte[] barr = blob.getData();

				System.out.println("TestBlob Known Length = "
						+ blob.getLength() + "\n START---");

				// TODO: Fix BLOB getLength()
				for (int i = 0; i < blob.getLength().intValue(); i++)
					System.out.print((char) barr[i]);
			} else {
				System.out.println("Not expected NULLLLLLL");
				assertTrue(false);
			}

			long t2 = System.currentTimeMillis();
			long tot = t2 - t1;
			System.out.println("--END: " + tot);

			assertTrue(true);
		} 
		catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}
		try {
			/**
			 * Possibility one (not knowing the length of the Blob):
			 */

			long t1 = System.currentTimeMillis();
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);
			op.readTuple(NdbOperation.LockMode.LM_Read);
			op.equal(1, 1);

			NdbBlob bh = op.getNdbBlobHandle(col2);

			trans.execute(ExecType.NoCommit,
					AbortOption.AbortOnError, true);
			System.out.println("Unknown length of Blob : " + bh.getLength());

			// TODO: Fix BLOB getLength()
			int len = bh.getLength().intValue();

			byte[] buff = new byte[len];
			bh.readData(buff, len);
			System.out.println("START---");
			for (int i = 0; i < len; i++)
				System.out.print((char) buff[i]);
			System.out.println("--END");
			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			long t2 = System.currentTimeMillis();
			long tot = t2 - t1;
			System.out.println("--END: " + tot);
			assertTrue(true);
		} 
		catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}
		try {

			/**
			 * knowing the length of the blob:
			 */
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);
			op.readTuple(NdbOperation.LockMode.LM_Read);
			op.equal(1, 1);
			long t1 = System.currentTimeMillis();
			NdbBlob bh = op.getNdbBlobHandle(col2);

			int len2 = 8000; // <--- knowledge!!!

			bh.getValue(len2);

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			if (!bh.getNull()) {
				byte[] barr = bh.getData();

				System.out.println("TestBlob Known Length = " + bh.getLength()
						+ "\n START---");

				// TODO: Fix BLOB getLength()
				for (int i = 0; i < bh.getLength().intValue(); i++)
					System.out.print((char) barr[i]);
			} else {
				System.out.println("Not expected NULLLLLLL");
				assertTrue(false);
			}
			long t2 = System.currentTimeMillis();
			long tot = t2 - t1;
			System.out.println("--END: " + tot);
			assertTrue(true);
		} catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

		try {
			/**
			 * update blob
			 */
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);


			System.out.println("Update Blob");

			op = trans.getNdbOperation(tablename);
			op.updateTuple();
			op.equal(col1, 2);

			NdbBlob bh = op.getNdbBlobHandle(col2);

			byte[] bb2 = { 'f', 'r', 'a', 'z', 'e', 'r', ' ', 'c', 'l', 'e',
					'm', 'e', 'n', 't', '4', '5' };


			trans.execute(ExecType.NoCommit,
					AbortOption.AbortOnError, true);

			bh.setValue(bb2);

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			assertTrue(true);
		} catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

		try {
			/**
			 * insert/delete blob
			 */
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);
			op.readTuple(NdbOperation.LockMode.LM_Read);
			op.equal(1, 1);

			NdbBlob bh = op.getNdbBlobHandle(col2);

			System.out.println("Insert/Delete Blob Test");
			long t1 = System.currentTimeMillis();
			trans = ndb.startTransaction();

			op = trans.getNdbOperation(tablename);
			op.insertTuple();
			op.equal("a", 9);
			bh = op.getNdbBlobHandle(col2);
			byte[] bb = { 'f', 'r', 'a', 'z', 'e', 'r', ' ', 'c', 'l', 'e',
					'm', 'e', 'n', 't', '2' };

			bh.setValue(bb);

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			trans.close();

			trans = ndb.startTransaction();

			op = trans.getNdbOperation(tablename);
			op.deleteTuple();
			op.equal("a", 9);

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			assertTrue(true);
		} 
		catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

		blob = null;
		try {
			long t1 = System.currentTimeMillis();
			trans = ndb.startTransaction();

			NdbIndexScanOperation op = trans.getNdbIndexScanOperation(
					"PRIMARY", tablename);

			op.readTuples(NdbOperation.LockMode.LM_Exclusive, 0, 0);
			op.setBoundInt(col1, NdbIndexScanOperation.BoundType.BoundLE, 1);
			op.getValue(col1);

			NdbResultSet rs = op.resultData();

			trans.execute(ExecType.NoCommit,
					AbortOption.AbortOnError, true);

			while (rs.next()) {
				int id = rs.getInt(col1);
				System.out.println("\nid=" + id);
				rs.deleteRow();
			}

			trans.execute(ExecType.Commit,
					AbortOption.AbortOnError, true);

			long t2 = System.currentTimeMillis();
			long tot = t2 - t1;
			System.out.println("--END: " + tot);

			assertTrue(true);
		} 
		catch (NdbApiException e) {
			System.out.println("Exception " + e.toString());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}
	}

}
