package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbError;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbScanFilter;
import com.mysql.cluster.ndbj.NdbScanOperation;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

/**
 *
create table t_multipart_primary (a int, b int, c varbinary(20), PRIMARY KEY(a,b)) engine=ndb;
insert into t_multipart_primary values(1,11,"jim.......");

 * @author jdowling
 *
 */
public class NdbIndexScanOperationUtf8Latin1Test extends BaseNdbjTestCase {

	protected static final String tablename = "t_multipart_primary";
	private static final String col1Val="jd";
	private static final String col2Val="jp";
	private static final String col3Val="jim";

	protected String col1="a";
	protected String col2="b";
	protected String col3="c";

	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbIndexScanOperationUtf8Latin1Test.class);
	}

	public NdbIndexScanOperationUtf8Latin1Test(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename, "(`a` VARCHAR(256) CHARSET utf8 NOT NULL, "
				+ "`b` VARCHAR(256) CHARSET utf8 NOT NULL, "
				+ "`c` VARCHAR(256), PRIMARY KEY(a,b), KEY (c)) ENGINE=ndb;");
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}


	public void testIndexScanOperationUtf8Latin1() {
	

		try {
			System.out.println("Starting a transaction for table " + tablename);
		    trans = ndb.startTransaction();

		    for (int i=0; i < 5; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);

				op.insertTuple();

				if (i > 0) {
					String pk1 = col1Val + i;
					op.equal(col1,pk1);
					String pk2 = col2Val + i;
					op.equal(col2,pk2);
				}
				else {
					op.equal(col1,col1Val);
					op.equal(col2,col2Val);
				}
				op.setString(col3,col3Val);

		    }
		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Inserted values(" + col1Val + ", " +
					col2Val + ", "
					+ col3Val + ");");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}
	
		try {
			System.out.println("Starting a setbound-eq index scan");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getNdbIndexScanOperation("PRIMARY",tablename);

			op.readTuples(NdbOperation.LockMode.LM_Exclusive,0,0);

			op.equal(col1,col1Val);
			op.equal(col2,col2Val);
			op.getValue(col3);

		    NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(rs.next())
			{
			    String c = rs.getString(col3);
			    System.out.println("Read latin1: c=" + c);
			}
		    op.close();
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting a setbound-eq index scan");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getNdbIndexScanOperation("c",tablename);

			op.readTuples(NdbOperation.LockMode.LM_Exclusive,0,0);

			op.equal(col3,col3Val);
			op.getValue(col1);
			op.getValue(col2);

		    NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(rs.next())
			{
			    String a = rs.getString(col1);
			    String b = rs.getString(col2);
			    System.out.println("Read latin1: a=" + a + ", b=" + b);
			}
		    op.close();

			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting a setbound-gt index scan");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbIndexScanOperation op =
				trans.getNdbIndexScanOperation("PRIMARY",tablename);

			op.readTuples(NdbOperation.LockMode.LM_Exclusive,0,0);

			op.setBoundString(col1,NdbIndexScanOperation.BoundType.BoundGE,col1Val);
			op.setBoundString(col2,NdbIndexScanOperation.BoundType.BoundGE,col2Val);
			op.getValue(col3);

		    NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(rs.next())
			{
			    String c = rs.getString(col3);
			    System.out.println("Read latin1: c=" + c);
			}
		    op.close();

			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
		    trans = ndb.startTransaction();

            NdbIndexScanOperation op =
                trans.getNdbIndexScanOperation("PRIMARY",tablename);
            
            op.readTuples(NdbOperation.LockMode.LM_Exclusive,0,0);
            
			op.equal(col1,col1Val);
			op.equal(col2,col2Val);
            
            NdbResultSet rs = op.resultData();

		    trans.execute(ExecType.NoCommit,
			      AbortOption.AbortOnError,
			      true);

            while (rs.next(true)) {
            	do {
                NdbOperation pkOp = rs.updateRow();
                pkOp.setString(col3, col3Val + "_jd");
                trans.execute(ExecType.Commit,
                        AbortOption.AbortOnError,
                        true);
            	} while (rs.next(false));
            }


            System.out.println("Finished update");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in update:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null) {
				trans.close();
			}
		}

		try {
		    trans = ndb.startTransaction();

            NdbIndexScanOperation op =
                trans.getNdbIndexScanOperation("PRIMARY",tablename);
            
            op.readTuples(NdbOperation.LockMode.LM_Exclusive,0,0);
			op.equal(col1,col1Val);
			op.equal(col2,col2Val);

            NdbResultSet rs = op.resultData();

            trans.execute(ExecType.NoCommit,
                      AbortOption.AbortOnError,
                      true);

            while (rs.next()) {
                rs.deleteRow();
            }
            trans.execute(ExecType.Commit,
                      AbortOption.AbortOnError,
                      true);

			System.out.println("Finished delete");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

	}

}
