package testsuite.ndbj;

import java.sql.Timestamp;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.NdbResultSet;


import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;


public class NdbOperationIDTest extends BaseNdbjTestCase {

    protected String col1="int_col";
    protected String col2="varcharlong_col";
    protected String col3="float_col";
    protected String col4="double_col";
    protected String col5="long_col";
    protected String col6="char_col";
    protected String col7="varchar_col";
    protected String col8="varbinary_col";
    protected String col9="varbinarylong_col";
    protected String col10="timestamp_col";
    protected String col11="binary_col";
    protected String col12="short_col";
    
	protected int col1ID=1;
	protected int col2ID=2;
	protected int col3ID=3;
	protected int col4ID=4;
	protected int col5ID=5;
	protected int col6ID=6;
	protected int col7ID=7;
	protected int col8ID=8;
	protected int col9ID=9;
	protected int col10ID=10;
	protected int col11ID=11;
    protected int col12ID=12;
    
	protected int 	int_val = 100;
	protected String varcharlong_val="Evelyn";
    // 240+ chars causes crash
	protected String varchar_val= "Paddy";//"0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789";
	protected byte[] varbinary_val="Maeve".getBytes(); // "0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789".getBytes();
	protected byte[] varbinarylong_val="Aoife".getBytes();
	protected byte[] binary_val="Rob".getBytes();
	protected String char_val="Linda";
	protected float float_val = 1.11F;
	protected double double_val = 2.22D;
	protected long long_val = 1233456789L;
	protected Timestamp timestamp_val = new Timestamp(System.currentTimeMillis());
    protected short short_val = 1974;
    
    protected final static String tablename = "t_ops";
    
	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbOperationIDTest.class);
	}

	public NdbOperationIDTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename, "(`"
				+ col1 + "` INT not null PRIMARY KEY, `"
				+ col2 + "` VARCHAR(256), `" 
				+ col3 + "` FLOAT, `"
				+ col4 + "` DOUBLE, `"
				+ col5 + "` BIGINT, `"
				+ col6 + "` CHAR(20), `"
                                + col7 + "` VARCHAR(255), `" // CHARACTER SET utf8
				+ col8 + "` VARBINARY(255), `"
				+ col9 + "` VARBINARY(256), `"
				+ col10 + "` TIMESTAMP, `"
                + col11 + "` BINARY(20), `"
                + col12 + "` SMALLINT"

				+ ") ENGINE=ndb;");
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

    
	public void testOperationID() {


		try {
			System.out.println("Starting a transaction for table " + tablename);
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbOperation op = trans.getNdbOperation(tablename);

			op.insertTuple();

			op.equal(col1,int_val);

			op.setString(col2,varcharlong_val);
			
			op.setFloat(col3, float_val);
			
			op.setDouble(col4, double_val);
			
			op.setLong(col5, long_val);
			
			// TODO: Fix character sets - this one is latin1
			op.setString(col6,char_val);
			
			// TODO: Fix character sets - this one is utf8
			op.setString(col7, varchar_val);
			
			op.setBytes(col8, varbinary_val);
			
			op.setBytes(col9, varbinarylong_val);
			
			// Timestamp objects stored in NDB lose nano-second accuracy
			timestamp_val.setNanos(0);

			op.setTimestamp(col10, timestamp_val);
			
			op.setBytes(col11, binary_val);
			
            op.setInt(col12, short_val);

			System.out.println("Going to execute Insert: ("+int_val+", "+varcharlong_val + ", " + float_val + ", " + double_val + ", " + long_val + ")");

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished insert");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			e.printStackTrace();
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting an read transaction");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbOperation op = trans.getNdbOperation(tablename);

		    op.readTuple(LockMode.LM_CommittedRead);

			op.equal(col1,int_val);
			op.getValue(col1);
			op.getValue(col2);
			op.getValue(col3);
			op.getValue(col4);
			op.getValue(col5);
			op.getValue(col6);
			op.getValue(col7);
			op.getValue(col8);
			op.getValue(col9);
			op.getValue(col10);
			op.getValue(col11);
            op.getValue(col12);
			

		    NdbResultSet rs = op.resultData();
		    System.out.println("Going to execute Read: id="+int_val+", name=?");

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			while(rs.next())
			{
				int id = rs.getInt(col1);
				assertEquals(id, int_val);
				
			    String name = rs.getString(col2);
			    assertEquals(name,varcharlong_val);  

			    float f = rs.getFloat(col3);
			    assertEquals(f,float_val);
			    
			    double d = rs.getDouble(col4);
			    assertEquals(d,double_val);
			    
			    long l = rs.getInt(col5);
			    assertEquals(l,long_val);
			    
			    String c = rs.getString(col6);
			    assertEquals(c,char_val);
			    
			    String vc = rs.getString(col7);
			    assertEquals(vc,varchar_val);
			    
			    byte[] vb = rs.getBytes(col8);
			    int k=0;
			    for (byte b: vb) {
			    	assertEquals(b,varbinary_val[k++]);
			    }			    
			    byte[] lvb = rs.getBytes(col9);
			    int m=0;
			    for (byte b: lvb) {
			    	assertEquals(b,varbinarylong_val[m++]);
			    }			    

			    Timestamp t = rs.getTimestamp(col10);
			    // ndb does not store timestamps to nanosecond accuracy
			    timestamp_val.setNanos(0);
			    assertEquals(t, timestamp_val);
			    
			    byte[] bin = rs.getBytes(col11);
			    int n=0;
			    for (byte b: bin) {
				    assertEquals(b,binary_val[n++]);
			    }			    

                short sh = rs.getShort(col12);
                assertEquals(sh,short_val);
                
			}


			System.out.println("Finished reading");
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Read Tuple:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
		    trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(tablename);

			op.updateTuple();

			op.equal(col1,int_val);
			
			varcharlong_val="jim";
			op.setString(col2,varcharlong_val);

			float_val += 4.111;
			op.setFloat(col3, float_val);
			
			double_val -= 5.000;
			op.setDouble(col4, double_val);
			
			long_val +=88888888;
			op.setLong(col5, long_val);
			
			// TODO: Fix character sets - this one is Latin1
			char_val="Grönqvist";
			op.setString(col6,char_val);
			
			// TODO: Fix character sets - this one is utf8
			varchar_val="Dowling";
			op.setString(col7, varchar_val);
			
			varbinary_val="Lyons".getBytes();
			op.setBytes(col8, varbinary_val);
			
			varbinarylong_val = "Lucan Sarsfields".getBytes();
			op.setBytes(col9, varbinarylong_val);
			
			timestamp_val.setTime(System.currentTimeMillis());
			op.setTimestamp(col10, timestamp_val);
			
			binary_val = "Spånga".getBytes();
			op.setBytes(col11, binary_val);
            
            short_val+=3;
            op.setInt(col12, short_val);
			
		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished update");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in update:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

		try {
			System.out.println("Starting an delete transaction");
		    trans = ndb.startTransaction();

			System.out.println("Getting an operation object from table:\t" +tablename);
			NdbOperation op = trans.getNdbOperation(tablename);

			op.deleteTuple();

			op.equal(col1ID,int_val);
		    System.out.println("Going to execute delete tuple, id="+int_val);

		    trans.execute(ExecType.Commit,
			      AbortOption.AbortOnError,
			      true);

			System.out.println("Finished delete");
			assertTrue(true);
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans!=null)
				trans.close();
		}

	}

}
