package testsuite.ndbj;

import java.io.UnsupportedEncodingException;
import java.sql.Timestamp;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.concurrent.CyclicBarrier;


import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.Ndb;
import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbBaseOperation;
import com.mysql.cluster.ndbj.NdbBlob;
import com.mysql.cluster.ndbj.NdbClusterConnection;
import com.mysql.cluster.ndbj.NdbError;
import com.mysql.cluster.ndbj.NdbIndexScanOperation;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbScanOperation;
import com.mysql.cluster.ndbj.NdbTransaction;
import com.mysql.cluster.ndbj.NdbOperation.LockMode;
import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class NdbTest extends BaseNdbjTestCase {

	static String PRIMARYKEYINDEXNAME = "PRIMARY";

	static String TABLE1 = "table1";
	static String TABLE2 = "table2";
	static String TABLE3 = "table3";
	static String key1   = "k1";
	static String key2   = "k2";
	static String data1  = "d1";
	static String data2  = "d2";
	static String data3  = "d3";
	static String data4  = "d4";
	static String data5  = "d5";
    static String data6  = "d6";
    static String data7  = "d7";

    static String rowTable1 = "rob";
    static String rowTable2 = "james";
    static String rowTable3 = "maeve";

	static final int TEST_BLOB_SIZE = 8*1024;
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(NdbTest.class);
	}

	public NdbTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(TABLE1,
				"( k1 VARCHAR(30) NOT NULL, d1 VARCHAR(20), "
						+ "d2 INTEGER, primary key (k1) ) engine=ndbcluster;");

		createTable(TABLE2,
				"( k1 VARCHAR(256) NOT NULL, k2 INTEGER NOT NULL, d1 VARCHAR(256),"
						+" d2 INTEGER, primary key (k1, k2) ) engine=ndbcluster;");

		createTable(TABLE3,
				"( k1 VARCHAR(256) NOT NULL, d1 TIMESTAMP, d2 CHAR(1),"
						+" d3 VARBINARY(100), d4 BLOB, d5 TEXT, d6 BIGINT,"
						+ "d7 BLOB, PRIMARY KEY (k1)) engine=ndbcluster;");

	}

	public void tearDown() throws Exception {
		super.tearDown();
	}
	

	private static void executeNoCommit( NdbTransaction trans ) throws NdbApiException
	{
		trans.execute( ExecType.NoCommit, AbortOption.AbortOnError, false);
	}

	private static void executeCommit( NdbTransaction trans ) throws NdbApiException
	{
		trans.execute( ExecType.Commit, AbortOption.AbortOnError, false);
	}

    private static void executeRollback( NdbTransaction trans ) throws NdbApiException
    {
        trans.execute( ExecType.Rollback, AbortOption.AbortOnError, false);
    }


	private static void printTable1( NdbResultSet rs ) throws NdbApiException
	{
		System.out.println("    Table1 results");

		while( rs.next() )
		{
			String  k1 = rs.getString(key1);
			String  d1 = rs.getString(data1);
			Integer d2 = rs.getInt(data2);

			System.out.printf("      Row: %-10s %-20s %10d \n", k1, d1, d2 );
		}

	}

	private static void printTable2( NdbResultSet rs ) throws NdbApiException
	{
		System.out.println("    Table2 results");

		while( rs.next() )
		{
			String  k1 = rs.getString(key1);
			Integer k2 = rs.getInt(key2);
			String  d1 = rs.getString(data1);
			Integer d2 = rs.getInt(data2);

			System.out.printf("      Row: %-10s %10s %-20s %10d \n", k1, k2, d1, d2 );
		}
	}


    private static String convertToHexString( byte[] bytes )
    {
        StringBuilder builder = new StringBuilder();

        for( byte b : bytes )
        {
            builder.append( Integer.toHexString(b) );
            builder.append(",");
        }

        return builder.toString();
    }



    private static String convertToHexString( byte[] bytes, int length )
    {
        StringBuilder builder = new StringBuilder();

        if( length > bytes.length )
        {
            builder.append( "Invalid length!\n");
            length = bytes.length;
        }

        for( int idx=0; idx<length; idx++ )
        {
            builder.append( Integer.toHexString(bytes[idx]) );
            builder.append(",");
        }

        return builder.toString();
    }





	private static void printNdbError( String source, NdbError error )
	{
		System.out.printf( "%15s Error: Status=%-10s Classification=%-10s Code=%5d Message=%s\n", source, error.getStatus(), error.getClassification(), error.getCode(), error.getMessage() );
	}

	/*
	 * Test method for 'com.mysql.ndbapi.Ndb.destroy()'
	 */

	public void testNdb(  )
	{
		System.out.println("\nTest: insertTable1");

		try {
			trans = ndb.startTransaction();

			NdbOperation op = trans.getNdbOperation(TABLE1);

			op.insertTuple();

			op.setString( key1,  rowTable1);
			op.setString( data1, null );
			op.setInt(    data2, 42 );

			executeNoCommit(trans);


			executeCommit(trans);

		}
		catch (NdbApiException e) {
			assertFalse(true);
		}
		finally {
			if (trans != null)
				trans.close();
		}
		
		
        System.out.println("\nTest: insertTable1 Expecting Error");

        NdbOperation op = null;

        try
        {
            trans = ndb.startTransaction();

            op = trans.getNdbOperation(TABLE1);

            op.insertTuple();

            op.setString( key1,  rowTable1 );
            op.setString( data1, "mynewvalue");
            op.setInt(    data2, 42 );

            executeNoCommit(trans);
            executeRollback(trans);

            assertFalse(true);
        }
        catch(NdbApiException e)
        {
            System.out.println("Caught Expected Exception");

    		assertTrue(true);
        }
		finally {
			if (trans != null)
				trans.close();
		}

        trans.close();
   
		System.out.println("\nTest: insertTable2");

		try {
			trans = ndb.startTransaction();

			NdbOperation op1 = trans.getNdbOperation(TABLE2);

			op1.insertTuple();

			op1.setString( key1,  rowTable2    );
			op1.setInt(    key2,  10          );
			op1.setString( data1, "mynewvalue");
			op1.setInt(    data2, 42          );

			NdbOperation op2 = trans.getNdbOperation(TABLE2);

			op2.insertTuple();

			op2.setString( key1,  rowTable2      );
			op2.setInt(    key2,  11            );
			op2.setString( data1, "myothervalue");
			op2.setInt(    data2, 5464          );

			executeNoCommit(trans);


			executeCommit(trans);
		}
        catch(NdbApiException e)
        {
            System.out.println("Caught Exception");
    		assertFalse(true);
        }
		finally {
			if (trans != null)
				trans.close();
		}




		System.out.println("\nTest: insertTable3");


		try {
		trans = ndb.startTransaction();

		op = trans.getNdbOperation(TABLE3);

		op.insertTuple();

		op.setString( key1, rowTable3 );

		Timestamp timestamp = new Timestamp( System.currentTimeMillis() );
		op.setTimestamp( data1, timestamp );


		op.setString( data2, "T" );

		byte[] smallbytes = new byte[60];
		for( int idx=0; idx<smallbytes.length; idx++ )
		{
			smallbytes[idx] = (byte)(idx + 10);
		}
		op.setBytes( data3, smallbytes );



        byte largeBytes[] = new byte[ TEST_BLOB_SIZE ];
        for( int idx=0; idx<largeBytes.length; idx++ )
        {
            largeBytes[idx] = (byte)(48+idx%10);
        }

        NdbBlob d4Blob = op.getNdbBlobHandle(data4);
        d4Blob.setValue( largeBytes );


        StringBuilder text = new StringBuilder();
        String testString = rowTable3 + ":Red Lorry, Yellow Lorry.";

        while( text.length() < TEST_BLOB_SIZE )
        {
            text.append(testString);
        }

        NdbBlob d5Blob = op.getNdbBlobHandle( data5 );
        try
        {
            d5Blob.setValue( text.toString().getBytes("Latin1") );
        }
        catch (UnsupportedEncodingException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }



		long longnum = 0x1234567890aabbccL;
		op.setLong( data6, longnum );

		System.out.println( timestamp.getTime() + " " + Long.toHexString(timestamp.getTime()));
		System.out.println( longnum + " " + Long.toHexString(longnum) );



        StringBuilder text2 = new StringBuilder();
        String testString2 = rowTable2+": Peter piper picked a peck of pickled pepper.";

        while( text2.length() < TEST_BLOB_SIZE )
        {
            text2.append(testString2);
        }

        System.out.println("d7 String length:"+text2.length());

        NdbBlob d7Blob = op.getNdbBlobHandle( data7 );
        try
        {
            d7Blob.setValue( text2.toString().getBytes("UTF8") );
        }
        catch (UnsupportedEncodingException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }



        executeNoCommit(trans);

		executeCommit(trans);

		assertTrue(true);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: readPrimary from Table1");

		try {
			trans = ndb.startTransaction();

			NdbOperation op1 = trans.getNdbOperation(TABLE1);

			op1.readTuple( NdbOperation.LockMode.LM_Exclusive );

			op1.equal(key1,rowTable1);

			op1.getValue(key1);
			op1.getValue(data1);
			op1.getValue(data2);

			NdbResultSet rs = op1.resultData();

			executeNoCommit(trans);

			printTable1(rs);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}

	
		System.out.println("\nTest: readPrimary from Table2");

		try {
			trans = ndb.startTransaction();


			NdbOperation op1 = trans.getNdbOperation(TABLE2);

			op1.readTuple( NdbOperation.LockMode.LM_Exclusive );

			op1.equal(key1,rowTable2);
			op1.equal(key2,10);

			op1.getValue(key1);
			op1.getValue(key2);
			op1.getValue(data1);
			op1.getValue(data2);

			NdbResultSet rs = op1.resultData();

			executeNoCommit(trans);

			printTable2(rs);

			executeCommit(trans);
			assertTrue(true);

		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}

        System.out.println("\nTest: Read Table3");

        try {
	        trans = ndb.startTransaction();

	        NdbOperation op1 = trans.getNdbOperation(TABLE3);

	        op1.readTuple( NdbOperation.LockMode.LM_Exclusive );

	        op1.equal( key1, rowTable3);
	        op1.getValue(data3);

	        NdbResultSet rs = op1.resultData();

	        executeNoCommit(trans);

	        if( rs.next() )
	        {
	            byte[] bytes = rs.getBytes(data3);

	            System.out.println("Bytes:"+convertToHexString(bytes));
	        }

	        executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: IndexScan using Primary for Table2");

		try {
			trans = ndb.startTransaction();

			NdbIndexScanOperation op1 = trans.getNdbIndexScanOperation(PRIMARYKEYINDEXNAME, TABLE2);

			op1.readTuples( NdbOperation.LockMode.LM_Exclusive, 0, 0 );

			op1.equal(key1,rowTable2);

			op1.getValue(key1);
			op1.getValue(key2);
			op1.getValue(data1);
			op1.getValue(data2);

			NdbResultSet rs = op1.resultData();

			executeNoCommit(trans);

			printTable2(rs);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("\nTest: scanAll Table1");

		try {
			trans = ndb.startTransaction();

			NdbScanOperation op1 = trans.getNdbScanOperation(TABLE1);

			op1.readTuples( NdbOperation.LockMode.LM_Exclusive, 0, 0 );

			op1.getValue(key1);
			op1.getValue(data1);
			op1.getValue(data2);

			NdbResultSet rs = op1.resultData();

			executeNoCommit(trans);

			printTable1(rs);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("\nTest: scanAll Table2");

		try {
			trans = ndb.startTransaction();

			NdbScanOperation op1 = trans.getNdbScanOperation(TABLE2);

			op1.readTuples( NdbOperation.LockMode.LM_Exclusive, 0,0);

			op1.getValue(key1);
			op1.getValue(key2);
			op1.getValue(data1);
			op1.getValue(data2);

			NdbResultSet rs = op1.resultData();

			executeNoCommit(trans);

			printTable2(rs);

			executeCommit(trans);
			assertTrue(true);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}

		System.out.println("\nTest: scanAll Table3");

		try {
			trans = ndb.startTransaction();

			NdbIndexScanOperation op1 = trans.getNdbIndexScanOperation(PRIMARYKEYINDEXNAME, TABLE3);

			op1.readTuples( NdbOperation.LockMode.LM_Exclusive, 0, 0);

			op1.getValue(key1);
			op1.getValue(data1);
			op1.getValue(data2);
			op1.getValue(data3);
			op1.getBlob(data4,TEST_BLOB_SIZE+1000);
			op1.getBlob(data5,TEST_BLOB_SIZE+1000);
			op1.getValue(data6);
	        op1.getBlob(data7,TEST_BLOB_SIZE+1000);

			NdbResultSet rs = op1.resultData();

			executeNoCommit(trans);

			System.out.println("    Table3 results");

			while( rs.next() )
			{
				// table3:
				// d1 TIMESTAMP,
				// d2 CHAR(1),
				// d3 VARBINARY(100),
				// d4 BLOB,
				// d5 TEXT,
				// d6 BIGINT

				String    k1 = rs.getString(key1);
				Timestamp d1 = rs.getTimestamp(data1 );
				String    d2 = rs.getString(   data2 );
				byte[]    d3 = rs.getBytes(    data3 );
				long      d6 = rs.getLong(     data6 );

	            System.out.println( "Timestamp long: " + Long.toHexString(d1.getTime()));

				NdbBlob   d4 = rs.getBlob(data4);

	            NdbBlob   d5 = rs.getBlob(data5);
	            String    d5str = null;

	            try
	            {
	            	// TODO: Fix blob getLength()
	                int len = d5.getLength().intValue();

	                byte[] d5Bytes = d5.getData();

	                System.out.println("bloblength:"+d5.getLength()+" array size:"+d5Bytes.length);

	                d5str = new String(d5Bytes, 0, d5.getLength().intValue(), "Latin1" );
	            }
	            catch (UnsupportedEncodingException e)
	            {
	                // TODO Auto-generated catch block
	                e.printStackTrace();
	            }

	            NdbBlob   d7 = rs.getBlob(data7);
	            String    d7str = null;

	            try
	            {
	            	// TODO: Fix blob getLength()
	                int len = d7.getLength().intValue();

	                byte[] d7Bytes = d7.getData();

	                d7str = new String(d7Bytes, 0, d7.getLength().intValue(), "UTF8" );

	                System.out.println("bloblength:"+d7.getLength()+" array size:"+d7Bytes.length+" string length:"+d7str.length());

	            }
	            catch (UnsupportedEncodingException e)
	            {
	                // TODO Auto-generated catch block
	                e.printStackTrace();
	            }


            	// TODO: Fix blob getLength()
	            int d4len = d4.getLength().intValue();
	            byte[] d4bytes = d4.getData();


				System.out.printf("      Row: %-10s '%tc' '%s' \nd3 (VarBinary):'%s' \nd4(blob)'%s' \nd5(Text)'%s' \nd6(long)'%d' \nd7(Text UTF8)'%s' \n",
	                              k1,
	                              d1,
	                              d2,
	                              convertToHexString( d3 ),
	                              convertToHexString( d4bytes, d4len ),
	                              d5str,
	                              d6,
	                              d7str
	                              );


			}

			executeCommit(trans);
			assertTrue(true);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: updateTable1");

		try {
			trans = ndb.startTransaction();

			op = trans.getNdbOperation(TABLE1);

			op.updateTuple();

			op.equal( key1, rowTable1 );

			op.setString( data1, "anothervalue");
			op.setInt(    data2, 24            );

			executeNoCommit(trans);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: updateTable2");

		try {
			trans = ndb.startTransaction();

			op = trans.getNdbOperation(TABLE2);

			op.updateTuple();

			op.equal( key1, rowTable2 );
			op.equal( key2, 10       );

			op.setString( data1, "yetanothervalue");

			executeNoCommit(trans);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: deleteTable1");

		try {
			trans = ndb.startTransaction();

			op = trans.getNdbOperation(TABLE1);

			op.deleteTuple();

			op.equal( key1, rowTable1 );

			executeNoCommit(trans);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: deleteTable2");

		try {
			trans = ndb.startTransaction();

			NdbIndexScanOperation scanOp = trans.getNdbIndexScanOperation( PRIMARYKEYINDEXNAME, TABLE2);

			scanOp.readTuples( NdbOperation.LockMode.LM_Exclusive, 0, 0 );

			scanOp.equal( key1, rowTable2 );

			NdbResultSet rs = scanOp.resultData();

			executeNoCommit(trans);


			while( rs.next() )
			{
				rs.deleteRow();
			}

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}


		System.out.println("\nTest: deleteTable3");

		try {
			trans = ndb.startTransaction();

			op = trans.getNdbOperation(TABLE3);

			op.deleteTuple();

			/**
			 * BUG: Delete on a row that doesn't exist with
			 * a Blob column causes an error
			 */
			op.equal( key1, rowTable3 );

			executeNoCommit(trans);

			executeCommit(trans);
		}
	    catch(NdbApiException e)
	    {
	        System.out.println("Caught Exception");
			assertFalse(true);
	    }
		finally {
			if (trans != null)
				trans.close();
		}

	}

}
