package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbTransaction;

import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class NullSetTest extends BaseNdbjTestCase {

	protected final static String tablename = "t_nulls";
	
	protected String col1="id";
	protected String col2="name";
	protected String col3="addr";
	protected String col4="size";

	public NullSetTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
		"(id VARBINARY(256) PRIMARY KEY, name VARBINARY(256), addr VARCHAR(256), size BIGINT) ENGINE=ndb;");

	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Test method for 'com.mysql.cluster.ndbj.NdbOperation.setNull(String)'
	 */
	public final void testNullSet() {


		try {
		    trans = ndb.startTransaction();
			byte[] name = null;
			int j=10;
				NdbOperation op = trans.getNdbOperation(tablename);
				op.insertTuple();
				byte[] pkID = "jim".getBytes();
				op.equal("id",pkID);
				op.setNull(col2);
				op.setNull(col3);
				op.setNull(col4);

			    trans.execute(ExecType.Commit,
				      AbortOption.AbortOnError,
				      true);
			}
			catch (NdbApiException e) {
				System.out.println("NdbApiException:\t" + e.getMessage());
				assertFalse(true);
			}
			finally {
				if (trans != null)
					trans.close();
			}



			try {
				    trans = ndb.startTransaction();
					NdbOperation op =trans.getNdbOperation(tablename);
					op.readTuple(NdbOperation.LockMode.LM_Exclusive);

					byte[] pkID = "jim".getBytes();
				    op.equal("id",pkID);
					op.getValue(col2);
					op.getValue(col3);
					op.getValue(col4);

				    NdbResultSet rs = op.resultData();

				    trans.execute(ExecType.Commit,
					      AbortOption.AbortOnError,
					      true);


			    while(rs.next())
				{
				    byte[] nm = rs.getBytes(col2);
				    if (rs.wasNull())
				    	System.out.println(col2 + " was null");
				    String addr = rs.getString(col3);
				    if (rs.wasNull())
				    	System.out.println(col3 + " was null");
				    long longVal = rs.getLong(col4);
				    if (rs.wasNull())
				    	System.out.println(col4 + " was null");
					System.out.println("\n"
							+ "\nname= " + nm.toString()
							+ ", \naddr=" + addr
							+ "\nnum=" + longVal);

				    System.out.println("Bytes:");
				    for (int i=0;i< nm.length; i++) {
				    	System.out.print(nm[i]);
				    	System.out.print(", ");
				    }
				    System.out.println();

				}

		}
		catch (NdbApiException e) {
			System.out.println("NdbApiException:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans != null)
				trans.close();
		}



		try {
		    trans = ndb.startTransaction();
			byte[] name = null;
			int j=10;
			System.out.println("Table is: " + tablename);
				NdbOperation op = trans.getNdbOperation(tablename);
				op.insertTuple();
				byte[] pkID = "tim".getBytes();
				op.equal("id",pkID);
				op.setNull(col2);
				op.setNull(col3);
				op.setNull(col4);

			    trans.execute(ExecType.Commit,
				      AbortOption.AbortOnError,
				      true);
			}
			catch (NdbApiException e) {
				System.out.println("NdbApiException:\t" + e.getMessage());
				assertFalse(true);
			}
			finally {
				if (trans != null)
					trans.close();
			}



			try {
				    trans = ndb.startTransaction();
					NdbOperation op =trans.getNdbOperation(tablename);
					op.readTuple(NdbOperation.LockMode.LM_Exclusive);

					byte[] pkID = "tim".getBytes();
				    op.equal(col1,pkID);
					op.getValue(col2);
					op.getValue(col3);
					op.getValue(col4);

				    NdbResultSet rs = op.resultData();

				    trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);


			    while(rs.next())
				{
				    byte[] nm = rs.getBytes(col2);
				    assertTrue(rs.wasNull());
				    
				    String addr = rs.getString(col3);
				    assertTrue(rs.wasNull());
				    long longVal = rs.getInt(col4);
				    assertTrue(rs.wasNull());
				    
				    System.out.println("Bytes:");
				    for (int i=0;i< nm.length; i++) {
				    	System.out.print(nm[i]);
				    	System.out.print(", ");
				    }
				    System.out.println();

				}

		}
		catch (NdbApiException e) {
			System.out.println("NdbApiException:\t" + e.getMessage());
			assertFalse(true);
		}
		finally {
			if (trans != null)
				trans.close();
		}


	}

}
