package testsuite.ndbj;

import testsuite.BaseNdbjTestCase;
import testsuite.TestCallback;
import com.mysql.cluster.ndbj.*;

public class QuickAsyncTest extends BaseNdbjTestCase {

	static boolean alreadySetUp = false;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// GENERAL STUFF
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	static int MAXIMUM_OPERATION_VOLUME       = 300; // TO DO: Calculate this

	static int NUMBER_OF_ELEMENT_INSERTIONS   = 25;

	static int NUM_REQUEST_TRANSACTIONS       = 2;
	static int REQUEST_TRANSACTIONS_TIMEOUT   = 1000;

	static int FORCE_SEND                     = 1;

	static boolean USE_SYNCHRONOUS            = false;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TABLE METADATA
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	static String tablename      = "t_quick_async";



	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// INITIALISATION CODE
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////


	public QuickAsyncTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();

		if (!alreadySetUp) {
			// Create required tables

			System.out.print("Creating tables...");

			this.createTable(tablename, 
				"    (ATTR1 INT UNSIGNED auto_increment not null," +
				"     ATTR2 INT UNSIGNED NOT NULL," +
				"     PRIMARY KEY (ATTR1) )" + 
				"  ENGINE=NDBCLUSTER");


			System.out.println("done.");
			System.out.print("Storing column indices...");

	

			alreadySetUp = true;
		}
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TEST CODE
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public void testSingleTx_Success() {
		
		try {
			trans = ndb.startTransaction();

			NdbOperation orderOp1 = trans.getNdbOperation(tablename);

			orderOp1.insertTuple();

			orderOp1.equalInt("ATTR1", 1);

			orderOp1.setInt("ATTR2", 1);
			
			BaseCallback callback = new TestCallback(1, ndb, trans, orderOp1.resultData());

			trans.executeAsynchPrepare(ExecType.Commit,
			                           callback,
			                           AbortOption.AbortOnError);

System.out.print("Performing sendPollNdb...");

			ndb.sendPollNdb(REQUEST_TRANSACTIONS_TIMEOUT, NUM_REQUEST_TRANSACTIONS, FORCE_SEND);

System.out.println("done.");
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			e.printStackTrace();
			assertFalse(true);
		}
		finally {
		}
	}

	public void testMultiTx_Failure() {
		
		try {
			trans = ndb.startTransaction();

			NdbOperation orderOp1 = trans.getNdbOperation(tablename);

			orderOp1.insertTuple();


			orderOp1.equalInt("ATTR1", 1);

			orderOp1.setInt("ATTR2", 1);


			BaseCallback callback = new TestCallback(1, ndb, trans, orderOp1.resultData());

			trans.executeAsynchPrepare(ExecType.Commit,
			                           callback,
			                           AbortOption.AbortOnError);

			trans = ndb.startTransaction();
			NdbOperation orderOp2 = trans.getNdbOperation(tablename);

			orderOp2.insertTuple();

			orderOp2.equalInt("ATTR1", 1);

			orderOp2.setInt("ATTR2", 1);

			TestCallback callback2 = new TestCallback(2, ndb, trans, orderOp2.resultData());

			trans.executeAsynchPrepare(ExecType.Commit,
			                           callback2,
			                           AbortOption.AbortOnError);

System.out.print("Performing sendPollNdb...");

			ndb.sendPollNdb(REQUEST_TRANSACTIONS_TIMEOUT, NUM_REQUEST_TRANSACTIONS, FORCE_SEND);

System.out.println("done.");
		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			e.printStackTrace();
			assertFalse(true);
		}
		finally {
		}
	}
	
	public void testOperation() {
		
		try {
			for (int loop = 0;
			loop < 100;
			loop++) {
				// Process request transaction

				NdbOperation orderOp;
				BaseCallback callback;

				for (int elementCount = 0;
				elementCount < NUMBER_OF_ELEMENT_INSERTIONS;
				elementCount++) {
					int id = (NUMBER_OF_ELEMENT_INSERTIONS * loop) + elementCount;

					trans = ndb.startTransaction();

					orderOp = trans.getNdbOperation(tablename);

					orderOp.insertTuple();


					orderOp.equalInt("ATTR1", id);

					orderOp.setInt("ATTR2", id);


					if (!USE_SYNCHRONOUS) {
						callback = new TestCallback(tablename, id, ndb, trans, orderOp.resultData());

						trans.executeAsynchPrepare(ExecType.Commit,
								callback,
								AbortOption.AbortOnError);
					}
				} 

				if (USE_SYNCHRONOUS) {
					trans.execute(ExecType.Commit, AbortOption.AbortOnError, true);
				}
				else {
					ndb.sendPollNdb(REQUEST_TRANSACTIONS_TIMEOUT, NUM_REQUEST_TRANSACTIONS, FORCE_SEND);
				}


			}

			System.out.println("Performing cooldown");

			ndb.sendPollNdb(100000, 100000, FORCE_SEND);	

		}
		catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Insert:\t" + e.getMessage());
			e.printStackTrace();
			assertFalse(true);
		}
		finally {
			if (trans!=null) {
				trans.close();
			}
		}
	}



}
