package testsuite.ndbj;

import java.util.Random;

import testsuite.BaseNdbjTestCase;

import junit.framework.AssertionFailedError;

import com.mysql.cluster.ndbj.NdbApiException;
import com.mysql.cluster.ndbj.NdbApiTemporaryException;
import com.mysql.cluster.ndbj.NdbApiUserErrorPermanentException;
import com.mysql.cluster.ndbj.NdbError;
import com.mysql.cluster.ndbj.NdbOperation;
import com.mysql.cluster.ndbj.NdbResultSet;
import com.mysql.cluster.ndbj.NdbTransaction;

import com.mysql.cluster.ndbj.AbortOption;
import com.mysql.cluster.ndbj.ExecType;

public class Utf8InsertReadTest extends BaseNdbjTestCase {

	protected static final String col1 = "id";

	protected static final String col2 = "name";

	protected static final String col2Val = "jimµ blah blah";

	protected static final int MAX_RETRIES = 3;

	protected static final int NUM_INSERTS = 2;

	protected static final String tablename = "t_utf8";
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(Utf8InsertReadTest.class);
	}

	public Utf8InsertReadTest(String arg0) {
		super(arg0);
	}

	public void setUp() throws Exception {
		super.setUp();
		createTable(tablename,
		"(`id` INT PRIMARY KEY, `name` VARCHAR(256) CHARSET utf8) ENGINE=ndb;");
	}

	public void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Test method for 'com.mysql.ndbapi.Ndb.destroy()'
	 */
	public void testUtf8InsertRead() {
	

		boolean done = false;
		int numRetries = 0;
		while (numRetries < MAX_RETRIES && !done) {
			try {
				trans = ndb.startTransaction();
				for (int i = 0; i < NUM_INSERTS; i++) {
					NdbOperation op = trans
							.getNdbOperation(tablename);
					op.insertTuple();
					op.equal(col1, i);
					op.setString(col2, col2Val);
				}

				trans.execute(ExecType.Commit,
								AbortOption.AbortOnError, true);

				done = true;
				assertTrue(true);
			} catch (NdbApiTemporaryException e) {
				numRetries++;
				System.out
						.println("Temporary error. Retrying insert. Num Retries == "
								+ numRetries + " error msg:" + e.getMessage());
			} catch (NdbApiException e) {
				System.out.println("Caught NdbApiException in Insert:\t"
						+ e.getMessage());
				assertTrue(false);
				done = true;
			} finally {
				if (trans != null)
					trans.close();
			}
		}

		try {
			System.out.println("Reading the inserted rows");
			trans = ndb.startTransaction();

			NdbResultSet[] rs = new NdbResultSet[NUM_INSERTS];
			for (int i = 0; i < NUM_INSERTS; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);
				op.readTuple(NdbOperation.LockMode.LM_Read);
				op.equal(col1, i);
				op.getValue(col2);
				rs[i] = op.resultData();
			}

			trans.execute(ExecType.Commit,
							AbortOption.AbortOnError, true);

			for (NdbResultSet r : rs) {
				while (r.next()) {
					String val = r.getString(col2);
					System.out.println("Read: " + val);
					if (val.compareTo(col2Val)!=0)
						assertFalse(true);
				}
			}

			System.out.println("Finished reading");
			assertTrue(true);
		} catch (NdbApiUserErrorPermanentException e) {
			System.out.println("User Permanent error. Retrying insert. Num Retries == "
							+ e.getMessage());
		} catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t"
					+ e.getMessage());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

	
		try {
			System.out.println("Deleting the inserted rows");
			trans = ndb.startTransaction();

			for (int i = 0; i < NUM_INSERTS; i++) {
				NdbOperation op = trans.getNdbOperation(tablename);
				op.deleteTuple();
				op.equal(col1, i);
			}

			trans.execute(ExecType.Commit,
							AbortOption.AbortOnError, true);

			System.out.println("Finished delete");
			assertTrue(true);
		} catch (NdbApiUserErrorPermanentException e) {
			System.out
					.println("User Permanent error. Retrying insert. Num Retries == "
							+ e.getMessage());
		} catch (NdbApiException e) {
			System.out.println("Caught NdbApiException in Delete:\t"
					+ e.getMessage());
			assertTrue(false);
		} finally {
			if (trans != null)
				trans.close();
		}

	}

}
