#
""" A Base Content Class """
debug=0
#from Contact import Person
#from Errors import * 
import re
import os
#from util import *
import ndb
if debug: print  ndb.__file__
import sys
import conns
import sqlobject

class MC(type):
    def __init__(cls, name, bases, dct):
        super(MC, cls).__init__(name, bases, dct)
        cls.columns={}
        for (k,v) in cls.__dict__.items():
            if hasattr(v,'__module__') and issubclass(v.__class__,sqlobject.Col):
##                if debug: print  cls.__name__,k
                v.name=k
                cls.columns[k]=v
        
        

class NdbObject(object):

    __metaclass__=MC
    
    content_registry = {}
    _content_mapping={}
    table_name = "base_content"
    _inheritable=True

    pre_create_hooks=[]
    post_create_hooks=[]
    pre_load_hooks=[]
    post_load_hooks=[]
    

    
    def __init__(self, id=None, create=False, **kw):

        
        self.__dict__['db'] = conns.__connection__
        
        if debug: print  "Columns = ",self.columns
        if create or id is None:
                #We need to create an id - probably in BaseContent
                for func in self.pre_create_hooks:
                    func(self,**kw)
                id=self.__class__._create(self,id,**kw)
                for func in self.post_create_hooks:
                    func(self,**kw)
                self.__dict__['id']=id
                
        else:
            # We want an existing one
            self.__dict__['id']=id
            self._load(self)

    def __setattr__(self, name, val):
        if debug: print  "__setattr__ called: %s|%s" % ( name, val )
        self.__dict__[name]=val
        self.__class__._set(self,name,val)



    @classmethod
    def _create(cls,self,id=None,class_name=None,**kw):

        bases=[ f for f in cls.__bases__ if f.__name__!='NdbObject']
        for superclass in bases:
            
            if hasattr(superclass,'_create'):
                if debug: print  "we are a child, creating parent",self.__class__.__name__
                id=superclass._create(self,id,class_name=self.__class__.__name__,**kw)
        
        if class_name is None:
            class_name=cls.__name__
        table=cls.table_name()
        if debug: print  "class: ",class_name
        if debug: print  "this class name:",cls.__name__


        try:
            trans = self.db.startTransaction()
            op = trans.getNdbOperation(cls.__name__)
        except RuntimeError, e:
            print e
            sys.exit(-1)

        op.insertTuple()
        if id is None:
          id=self.db.getAutoIncrementValue(class_name,5)

        if debug: print  "id: ",id
        op.equal("id",id)
        cols=[]
        vals=[]
        if cls._inheritable:
            op.setValue("class_name",class_name)
            
        for f in cls.columns.values():
            name=f.name
            if kw.has_key(name):
                val = kw[name]
                # Special case for foreign keys - not truly handled yet
                if isinstance(f,sqlobject.ForeignKey):
                    name="%s_id" % name
                op.setValue(name,val)

                self.__dict__[f.name]=kw[f.name]

        try: 
            trans.execute(ndb.Commit)
            self.db.closeTransaction(trans)
        except RuntimeError, e:
            print e
            sys.exit(-1)

        return id
    
    @classmethod
    def _load(cls,self):
        
        bases=[ f for f in cls.__bases__ if f.__name__!='NdbObject']
        for superclass in bases:
            
            if hasattr(superclass,'_load'):
                if debug: print  "we are a child, loading parent",self.__class__.__name__
                superclass._load(self)

        table=cls.table_name()
        if debug: print "table: " , table

        trans = self.db.startTransaction()
        op = trans.getNdbOperation(table)
        op.readTuple(ndb.NdbOperation.LM_Read)

        op.equal('id',self.id)

        myRecAttrs={}
        if debug: print  "cls.columns: %s" % cls.columns
        for f in cls.columns.values():
            name=f.name
            if isinstance(f,sqlobject.ForeignKey):
                name="%s_id" % name
            if debug: print  "getting value %s" % name
            myRecAttrs[name]=op.getValue(name)
            if myRecAttrs[name] is None:
                raise op.getNdbError().getMessage()
                
        if (trans.execute(ndb.Commit)==-1):
            if debug: print trans.getNdbError().getMessage()

        for key in myRecAttrs.keys():
            val = myRecAttrs[key].get_value()
            if debug: print "got: %s|%s" % (key,val)
            self.__dict__[key]=myRecAttrs[key].get_value()
        


    @classmethod
    def _set(cls,obj,name,val):

        # first try my own columns
        my_cols =  [f.name for f in cls.columns.values()]
        if debug: print  my_cols
        if cls.columns.has_key(name):

            col = cls.columns[name]
            if isinstance(col,sqlobject.ForeignKey):
                #We have two choices. Either we're passing in the object
                # or the id.
                print col._name


            
            trans = obj.db.startTransaction()
            table=cls.table_name()
            
            
            if debug: print  trans, "table: ", table
            
            op = trans.getNdbOperation(table)
            if debug: print  op, obj.__dict__['id']
            if op.writeTuple()==-1:
                if debug: print  op.getNdbError().getMessage()
            if op.equal("id",obj.__dict__['id'])==-1:
                if debug: print  op.getNdbError().getMessage()
            if debug: print  "op::setting %s|%s" % (name,val)
            if op.setValue(name,val)==-1:
                if debug: print  op.getNdbError().getMessage()
            if (trans.execute(ndb.Commit)==-1):
                #handle error
                if debug: print  trans.getNdbError().getMessage()
            obj.db.closeTransaction(trans)
            return 0

        # Didn't have it locally - let's try the parent
        for superclass in cls.__bases__:
            if debug: print  "trying parent: ", superclass.__name__
            if hasattr(superclass,'_set'):
                ret = superclass._set(obj,name,val)
                if ret == 0:
                    return 0
        raise "couldn't set value"
         
        #if name in cls.simple_fields:
        #    c = obj.db.cursor()
        #    query = "update %s set %s=%%s where id=%%s" % (cls.table_name(),name)
        #    c.execute(query, (val, obj.id))
        
    @classmethod
    def table_name(cls):
        return cls.__name__

    @classmethod
    def get(cls, db, select_phrase=None, order_by="order by position", **kw):

        ret=[]
        c = db.cursor()
        and_class_name=''
        if cls._inheritable:
            and_class_name = ", class_name"
        select_sql="select id %s from %s  where " % (and_class_name,cls.table_name())
        predicates = []
        pred_vals = []
        for k,v in kw.items():
            predicates.append("%s=%%s" % (k))
            pred_vals.append(v)
            #if type(v)==type(0):
            #    predicates.append("%s=%%s" % (k))
            #else:
            #    pred_vals.append(db.escape_string(str(v)))
        if select_phrase:
            predicates.append("( %s )" % select_phrase)
        select_sql="%s %s %s " % (select_sql," and ".join(predicates),order_by)
        c.execute(select_sql,pred_vals)
        results=c.fetchall()

        columns=[f[0] for f in c.description]
        for row in results:
            row_dict = to_dict(c, row)
            if 'class_name' in columns:
                class_name = row_dict['class_name']
                k=BaseContent._content_mapping[class_name]
            else:
                k=cls
            ret.append(k(db, id=row_dict['id']))
        return ret

    @classmethod
    def select(cls, db, select_phrase):
        return cls.get(db,select_phrase=select_phrase)
    
    def _getParent(self):
        return BaseContent.get(self.db,id=self.parent_id)[0]

    parent=property(_getParent, None, None, "Parent Object")

    def getContentItems(self):
        """ Return a list of contained ContentObjects """
        return BaseContent.get(self.db,parent_id=self.id)

    def getParentContentItems(self):
        """ Return a list of the sibling ContentObjects """
        return BaseContent.get(self.db,parent_id=self.parent.id)

    def raiseImageCount(self):
        """ Increases the image count in the current object and all
        parent objects"""
        self.image_count=self.image_count+1
        self.parent.raiseImageCount()

    def getNamePath(self):
        return "/".join([self.parent.getNamePath(),self.name])

    def getBreadCrumbs(self):
        bc=list(self.parent.getBreadCrumbs())
        bc.append(self)
        return bc

    def getViewableChildren(self):
        return BaseContent.select(self.db,"parent_id=%s and image_count>0 and class_name!='Image'" % self.id)

    def getImages(self):
        return BaseContent.select(self.db,"parent_id=%s and class_name='Image'" % self.id)

    def getFirstImage(self):
        imgs=list(BaseContent.select(self.db,"parent_id=%s and image_count>0" % self.id))

        return imgs[0].getFirstImage()

    def getNextPosition(self):

        # we select directly from BaseContent here
        c=self.db.cursor()
        c.execute("""select position from BaseContent where
        parent_id=%s order by position desc limit 1""" % self.id)
        results = c.fetchone()
        if debug: print  results
        
        return results[0]+1
        

    def processStack(self,request,url_stack=[]):
        """ Continue to walk the stack - either returns view or the output
        of it's children """

        if len(url_stack) > 0:
            # If it's the end of the stack, see if it's a method we have
            if len(url_stack) == 1:
                if hasattr(self,url_stack[0]):
                    meth=getattr(self,url_stack[0])
                    return meth(request)
            # It's not, let's see if it's a child
                
            child = self.getChild(url_stack[0])
            if child is not None:
                self.child=child
                if len(url_stack) == 0:
                    # The child will be the end...
                    return child.processStack(request,[])
                else:
                    return child.processStack(request,url_stack[1:])

                # We don't know what kind of crack you're smoking
            raise NotFoundException

        # Otherwise - return us
        return self

            

    def getChildByID(self, tree_id):
        """ Get a child object based on an ID.
        Returns a child object or None """

        _get_child_id = """
        select name, type, position
        from tree
        where id=%s
        """
        cursor=self.db.cursor()
        if (cursor.execute(_get_child_id,(tree_id)) != 1):
            # <TODO> Throw exception on too many rows
            return None
        (name,type,position)=cursor.fetchall()[0]
        #if debug: print  "position: %s\tname: %s\ttree_id: %s" % (position,name,tree_id)
        return self._content_mapping[type](self, self.db, name, tree_id, position)

    def getChild(self, name):
        """ Get a child object based on a name.
        Returns a child object or None """


        _get_child_id = """
        select id, type, position
        from tree
        where parent_id=%s
        and name=%s
        """

        cursor=self.db.cursor()
        if (cursor.execute(_get_child_id,(self.tree_id,name)) != 1):
            # <TODO> Throw exception on too many rows
            return None
        (tree_id,type,position)=cursor.fetchall()[0]
        if debug: print  "position: %s\tname: %s\ttree_id: %s" % (position,name,tree_id)
        return self._content_mapping[type](self, self.db, name, tree_id, position)

    def getChildren(self):
        """ Get a list of child objects. """
        _get_children = """
        select id, type, position, name
        from tree
        where parent_id=%s
        """
        children=[]
        cursor=self.db.cursor()
        cursor.execute(_get_children,(self.tree_id,))
        for (tree_id, type, position, name) in cursor.fetchall():
            children.append(self._content_mapping[type](self, self.db, name, tree_id, position))
        self.children=children
        return children

    def getName(self):
        if hasattr(self, 'username'):
            return self.username
        return self.name

    def _call_template(self, template_name, request=None):
        (context, template, out) = request.getTemplate(template_name)
        context.addGlobal('request',request)
        context.addGlobal('here',self)
        template.expand(context,out)
        
        return out.getvalue()

    def view(self, request=None):
        return self._call_template(self.view_template, request)

    def edit(self, request=None):
        return self._call_template(self.edit_template, request)

    def addPortfolio(self, request=None):
        request.debug("I'm in addPortfolio")
        name=request.fields.get("name")
        request.db.newPortfolio(self.getTreeId(),name)
        return "+:%s" % name

    
    def getTreeId(self):
        return self.tree_id

    def getTemplate(self, template):
        if template=="view":
            return self.view_template
        elif template=="edit":
            return self.edit_template
    
    def foo(self):
        return "foo"
    
    def moveUp(self, request):
        return self._move(-1)
        
    def moveDown(self, request):
        return self._move(1)

    def _move(self, offset):
        _get_pos_id = "select id from tree where parent_id=%s and position=%s"
        _set_pos = "update tree set position=%s where id=%s"
        cursor=self.db.cursor()
        if self.parent is None:
            parent_id=0
        else:
            parent_id=self.parent.getTreeId()
            
        if (cursor.execute(_get_pos_id, (parent_id,self.position+offset))==1):
            other_row_id = cursor.fetchall()[0][0]
            
            # We only do it if there is another item to swap with
            cursor.execute(_set_pos,(self.position,other_row_id))
            cursor.execute(_set_pos,(self.position+offset,self.tree_id))
        else:
            raise "Too many or too few rows with the same position"
        return 0

    def _makeName(self, title):
        title = title[:26]
        name = re.sub('\W','_',title)
        name = re.sub('_+','_',name)
        return name

    def newObject(self, typ, title, name=None):
        if not name:
            name = self._makeName(title)
        else:
            name = self._makeName(name)
        cursor=self.db.cursor()
        parent_id=self.getTreeId()
        siblings = self.getChildren()
        position = len(siblings)+1
        nameadd = ''
        while 1:
            ok = 1 
            for sibling in siblings:
                if sibling.name==name+nameadd:
                    ok = 0
                    break
            if not ok:
                if nameadd: nameadd = str(int(nameadd)+1)
                else: nameadd = '2'
            if ok:
                break
                
        cursor.execute("""insert into tree
                            (parent_id, type, name, title, position)
                          values (%s, %s, %s, %s, %s)""",
                       (parent_id, typ, name, title, position))
        return cursor.lastrowid

    def fullPath(self):
        """ Includes the username at the beginning -- for filesystem. """
        p = self
        path = p.name
        while p.parent:
            p = p.parent
            path = os.path.join(p.getName(), path)
        return path

    def contentPath(self):
        """ Does not include the username -- for web. """
        p = self
        path = p.name
        while p.parent:
            p = p.parent
            if hasattr(p, 'username'):
                break
            path = os.path.join(p.getName(), path)
        return path


#BaseContent._build_cols()

#import Portfolio, Project, Image, Member

#BaseContent._content_mapping = {
##    'person': Member.Member,
#    'Portfolio': Portfolio.Portfolio,
#    'Project': Project.Project,
#    'Image': Image.Image,
#    }
