// -*- mode: c++ -*- 
/*  ndb-connectors: Wrappers for the NDBAPI
    Copyright (C) 2006 MySQL, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

%module ndbapi

%include "ndbapi/ndbglobals.i"



%typemap(in) asynch_callback_t * cb {
  
  if (!PyCallable_Check($input)) {
    PyErr_SetString(PyExc_TypeError, "Need a callable object!");
    return NULL;
  }
  $1 = new asynch_callback_t;
  $1->obj = $input;
  Py_INCREF($input);
  $1->create_time=0;
 }

%typemap(in) (MYSQL_TIME *) { 
  if (PyDateTime_Check($input)) { 
    MYSQL_TIME * dt = (MYSQL_TIME *)malloc(sizeof(MYSQL_TIME));
    if (dt==NULL) 
      NDB_exception(NdbApiException,"Failed to allocate a MYSQL_TIME");
    dt->year = PyDateTime_GET_YEAR($input);
    dt->month = PyDateTime_GET_MONTH($input);
    dt->day = PyDateTime_GET_DAY($input);
    dt->hour = PyDateTime_DATE_GET_HOUR($input);
    dt->minute = PyDateTime_DATE_GET_MINUTE($input);
    dt->second = PyDateTime_DATE_GET_SECOND($input);
    $1 = dt;      
  } else { 
    NDB_exception(NdbApiException,"DateTime argument required!");
  }
 }

%typemap(in) (const char* anInputString, size_t len) {
  /* Check that we are getting a string */
  if (PyString_Check($input)) {
    // We are going to try not copying this string, since it's not going to 
    // be modified and we're going to copy it in the main function
    $1=PyString_AsString($input); 
    $2=PyString_Size($input);
  } else { 
    NDB_exception(NdbApiException,"Couldn't convert argument");
  }
 }

%typemap(freearg) (MYSQL_TIME *) {
  free((MYSQL_TIME *) $1);
 }


%runtime %{

// Include python/datetime.h



  #include <datetime.h>

  
  %}


%{
  

typedef struct
asynch_callback_t
{  
  PyObject *obj;
  long long create_time;
};

/* This function matches the prototype of the normal C callback
   function for our widget. However, we use the clientdata pointer
   for holding a reference to a Python callable object. */
  
  
  static void theCallBack(int result,
			  NdbTransaction *trans,
			  void *aObject)
  {
    PyObject *func, *arglist;
    PyObject *trans_obj = 0;
    
    asynch_callback_t * callback_data = (asynch_callback_t *)aObject;
    func = (PyObject *) callback_data->obj;   // Get Python callable
    trans_obj = SWIG_NewPointerObj(SWIG_as_voidptr(trans), SWIGTYPE_p_NdbTransaction, 0 |  0 );
    arglist = Py_BuildValue("(i,O)",result, trans_obj);             // Build argument list
    delete callback_data; 
    PyEval_CallObject(func,arglist);     // Call Python
    Py_DECREF(func);
    Py_DECREF(arglist);                  // Trash arglist
    
  }
  
#define NEW_pyexcept(EXCPT,EPARENT) { PyExc_ ## EXCPT = PyErr_NewException("mysql.cluster.ndbapi." #EXCPT ,EPARENT,NULL); PyDict_SetItemString(d, #EXCPT ,PyExc_ ## EXCPT); Py_INCREF(PyExc_ ## EXCPT); }
  
  PyObject * PyExc_NdbApiException; 
  PyObject * PyExc_BlobUndefinedException;
  PyObject * PyExc_NdbApiPermanentException;
  PyObject * PyExc_NdbApiRuntimeException;
  PyObject * PyExc_NdbApiTemporaryException;
  PyObject * PyExc_NdbApiTimeStampOutOfBoundsException;
  PyObject * PyExc_NdbApiUserErrorPermanentException;
  PyObject * PyExc_NdbClusterConnectionPermanentException;
  PyObject * PyExc_NdbClusterConnectionTemporaryException;
  PyObject * PyExc_NoSuchColumnException;
  PyObject * PyExc_NoSuchIndexException;
  PyObject * PyExc_NoSuchTableException;
  
#define NDB_exception(excp, msg) { ndb_throw_exception(excp,msg); SWIG_fail; }
// TODO: Need to support this form of exception
#define NDB_exception_err(excp, msg, err) { ndb_throw_exception(excp,msg); SWIG_fail; }
  
  void ndb_throw_exception(NdbException excp_mod, const char * msg) { 
    
    PyObject * exception; 
    
    switch (excp_mod) { 
    case NdbApiException:
      exception = PyExc_NdbApiException;
      break; 
    case BlobUndefinedException:
      exception = PyExc_BlobUndefinedException;
      break;
    case NdbApiPermanentException:
      exception = PyExc_NdbApiPermanentException;
      break;
    case NdbApiRuntimeException:
      exception = PyExc_NdbApiRuntimeException;
      break;
    case NdbApiTemporaryException:
      exception = PyExc_NdbApiTemporaryException;
      break;
    case NdbApiTimeStampOutOfBoundsException:
      exception = PyExc_NdbApiTimeStampOutOfBoundsException;
      break;
    case NdbApiUserErrorPermanentException:
      exception = PyExc_NdbApiUserErrorPermanentException;
      break;
    case NdbClusterConnectionPermanentException:
      exception = PyExc_NdbClusterConnectionPermanentException;
      break;
    case NdbClusterConnectionTemporaryException:
      exception = PyExc_NdbClusterConnectionTemporaryException;
      break;
    case NoSuchColumnException:
      exception = PyExc_NoSuchColumnException;
      break;
    case NoSuchIndexException:
      exception = PyExc_NoSuchIndexException;
      break;
    case NoSuchTableException:
      exception = PyExc_NoSuchTableException;
      break;
    default:
      exception = PyExc_RuntimeError;
    }
    exception = PyExc_NdbApiException;
    PyErr_SetString(exception,msg);
    Py_INCREF(exception);  
  }
  
  
  
  %}

%include "ndbapi/NdbFactory.i"

%include "ndbapi/NdbClusterConnection.i"


%include "ndbapi/Ndb.i"
%include "ndbapi/NdbDictionary.i"

%include "ndbapi/NdbTransaction.i" 



%init %{


  printf("calling ndb_init()\n");
  ndb_init();
  printf("after calling ndb_init()\n");
  
  PyDateTime_IMPORT;

  NEW_pyexcept(NdbApiException,NULL);
  NEW_pyexcept(NdbApiRuntimeException,PyExc_NdbApiException);
  NEW_pyexcept(NdbApiPermanentException,PyExc_NdbApiException);
  NEW_pyexcept(NdbApiTemporaryException,PyExc_NdbApiException);
  NEW_pyexcept(NdbApiUserErrorPermanentException,PyExc_NdbApiPermanentException);
  NEW_pyexcept(NdbClusterConnectionPermanentException,PyExc_NdbApiPermanentException);
  NEW_pyexcept(NdbClusterConnectionTemporaryException,PyExc_NdbApiTemporaryException);
  NEW_pyexcept(NdbApiTimeStampOutOfBoundsException,PyExc_NdbApiException);
  NEW_pyexcept(BlobUndefinedException,PyExc_NdbApiException);
  NEW_pyexcept(NoSuchColumnException,PyExc_NdbApiException);
  NEW_pyexcept(NoSuchIndexException,PyExc_NdbApiException);
  NEW_pyexcept(NoSuchTableException,PyExc_NdbApiException);

%}

%include "ndbapi/NdbOperation.i"

%include "ndbapi/NdbScanOperation.i"
%include "ndbapi/NdbIndexOperation.i"
%include "ndbapi/NdbIndexScanOperation.i"
%include "ndbapi/NdbEventOperation.i"


%include "ndbapi/NdbRecAttr.i"

%extend NdbRecAttr {

  %pythoncode %{
def getValue(self):
  t=self.getColType()

  if t >= NDB_TYPE_TINYINT and t <= NDB_TYPE_BIGINT and (t%2==0.0):
    ret = self.getInt32()
  elif t >= NDB_TYPE_TINYUNSIGNED and t <= NDB_TYPE_BIGUNSIGNED and (t%2==1):
    ret = self.getUint32()
  elif t == NDB_TYPE_FLOAT:
    ret = self.getFloat()
  elif t == NDB_TYPE_DOUBLE or t == NDB_TYPE_OLDDECIMAL:
    ret = self.getDouble()
  elif t >= NDB_TYPE_CHAR and t <= NDB_TYPE_VARBINARY:
    ret = self.aRef()
  elif t == NDB_TYPE_DATE: 
    ret = self.getUint32()
    year = ret/10000 % 10000
    month = ret/100 % 100
    day = ret % 100
    ret = "%s-%s-%s" % (year,month,day)
  else:
    # TODO add newer datatypes here
    raise NdbApiException, "unknown type"
  return ret
  
    %}

};

%include "ndbapi/NdbError.i"
%include "ndbapi/NdbBlob.i"
%include "ndbapi/NdbScanFilter.i"


%pythoncode %{ 

NdbApiException=_ndbapi.NdbApiException
NdbApiPermanentException=_ndbapi.NdbApiPermanentException
BlobUndefinedException=_ndbapi.BlobUndefinedException
NdbApiRuntimeException=_ndbapi.NdbApiRuntimeException
NdbApiTemporaryException=_ndbapi.NdbApiTemporaryException
NdbApiTimeStampOutOfBoundsException=_ndbapi.NdbApiTimeStampOutOfBoundsException
NdbApiUserErrorPermanentException=_ndbapi.NdbApiUserErrorPermanentException
NdbClusterConnectionPermanentException=_ndbapi.NdbClusterConnectionPermanentException
NdbClusterConnectionTemporaryException=_ndbapi.NdbClusterConnectionTemporaryException
NoSuchColumnException=_ndbapi.NoSuchColumnException
NoSuchIndexException=_ndbapi.NoSuchIndexException
NoSuchTableException=_ndbapi.NoSuchTableException

def connect(connectstring=None,no_retries=0,retry_delay_in_seconds=1,
            verbose=0,timeout_for_first_alive=1,timeout_after_first_alive=1,*args,**kwargs):
  """ Provide DBAPI 2.0 support """ 
  connection=NdbFactory.createNdbClusterConnection(connectstring)
  connection.connect(no_retries,retry_delay_in_seconds,verbose)
  connection.wait_until_ready(timeout_for_first_alive,timeout_after_first_alive)
  return connection
  
%}

/*
 Turns out we don't need these, but I'm leaving them here for reference.
#if defined(SWIGPYTHON)
%typemap(out) Uint32 {
  $result = PyInt_FromUnsignedLong((long) $1);
 }
%typemap(out) Int32 {
  $result = PyInt_FromLong((long) $1);
 }
%typemap(out) Uint64 {
  $result = PyLong_FromUnsignedLongLong((long) $1);
 }
%typemap(in) Int32 {
  $1 = (Int32) PyLong_AsLong($input);
 }
%typemap(in) Uint32 {
  if ($input < 0) {
     PyErr_SetString(PyExc_ValueError,"Expected a nonnegative value.");
     return NULL;
  }
  $1 = (Uint32) PyLong_AsUnsignedLong($input);
 }
%typemap(in) Uint64 {
  if ($input < 0) {
     PyErr_SetString(PyExc_ValueError,"Expected a nonnegative value.");
     return NULL;
  }
  $1 = (Uint64) PyLong_AsUnsignedLong($input);

 }
%typemap(typecheck,precedence=SWIG_TYPECHECK_INTEGER) Uint32 {
   $1 = PyInt_Check($input) ? 1 : 0;
}
%typemap(typecheck,precedence=SWIG_TYPECHECK_INTEGER) Uint64 {
   $1 = PyLong_Check($input) ? 1 : 0;
}
%typemap(typecheck,precedence=SWIG_TYPECHECK_INTEGER) Int32 {
   $1 = PyInt_Check($input) ? 1 : 0;
}
#endif

*/
