import sys,time,random
from mysql.cluster import ndbapi
import MySQLdb

if len(sys.argv) != 3:
  print "Usage:\n\ttest.py NUM_OF_ITERATIONS NUM_OF_ROWS "
  sys.exit(1)

num_iter=sys.argv[1]
INSERT_NUM=sys.argv[2]

db = MySQLdb.connect(host="localhost",user="root", read_default_group="client")

cur=db.cursor()

print "Dropping and recreating schema\n"

cur.execute("CREATE DATABASE if not exists test")
cur.execute("USE test")

cur.execute("""
CREATE TABLE if not exists
  mytablename
   (ATTR1 INT UNSIGNED,
    ATTR2 varchar(32) not null,
    PRIMARY KEY USING HASH (ATTR1),
  UNIQUE MYINDEXNAME USING HASH (ATTR2))
ENGINE=NDBCLUSTER
""")

db.close()


### Connect to cluster

print "connecting to cluster\n"
    
connection = ndbapi.NdbFactory.createNdbClusterConnection() 
#NdbClusterConnection()

if (connection.connect(5,3,1)):

    print "Connect to cluster management server failed."
    sys.exit(-1)
    
    
if (connection.waitUntilReady(30,30)):
    print "Cluster was not ready within 30 secs."
    sys.exit(-1);


myNdb = ndbapi.Ndb(connection,"test" )

if (myNdb.init(4) == -1):
    print myNdb.getNdbError()
    sys.exit(-1)


print "running tests:\n"

### Fill db


before_t = time.time()

for i in range(0,int(INSERT_NUM)):

    myTransaction= myNdb.startTransaction()
      
    myOperation= myTransaction.getNdbOperation("mytablename")
      
    myOperation.insertTuple()
      
    auto_id = myNdb.getAutoIncrementValue("mytablename",0)
        
    print auto_id
    myOperation.equalUint64("ATTR1",auto_id);
    myOperation.setValue("ATTR2", i);
      
    ret = myTransaction.execute( ndbapi.Commit )
    if ret == -1:
	print myTransaction.getNdbError().message
        
    myTransaction.close()

after_t = time.time()
print "Insert time for %s: %s -- %s" % ( INSERT_NUM, after_t, before_t)
print "\t %s" % (after_t - before_t)


### Get list of ids
    
myTransaction = myNdb.startTransaction()


myScanOperation=myTransaction.getNdbScanOperation("mytablename");


if myScanOperation is None:
    print myTransaction.getNdbError();


if myScanOperation.readTuples(ndbapi.NdbOperation.LM_CommittedRead):
    print myScanOperation.getNdbError();


    
myRecAttr=myScanOperation.getValue("ATTR1");
    
    
myTransaction.execute(ndbapi.NoCommit);

ids=[]
while 1:
      
    if (myScanOperation.nextResult(True) != 0) :
        break
      
    random_id = myRecAttr.getValue()
    ids.append(random_id)
      
    
myTransaction.close()


### Fetch ids using ndbapi

before_t = time.time()

for f in range(0,int(num_iter)):

    myTrans = myNdb.startTransaction()
    if myTrans is None:
        print myNdb.getNdbError()

    myOper = myTrans.getNdbOperation("mytablename")
    myOper.readTuple(ndbapi.NdbOperation.LM_Read)

    id_num = ids[random.randrange(0,len(ids))]
    myOper.equalUint64("ATTR1",id_num)
            
    myRecAttr= myOper.getValue("ATTR2")

    
    if myTrans.execute( ndbapi.Commit ) == -1:
        print myTrans.getNdbError()
    #foo=myRecAttr.getValue()
    foo=myRecAttr.getValue()
    print foo, myRecAttr.getColType(), myRecAttr.getValue()
    myTrans.close()
      

after_t = time.time()
print "NDBAPI time for %s: %s -- %s" % ( num_iter, after_t, before_t)
print "\t %s" % (after_t - before_t)


db = MySQLdb.connect(host="localhost",user="root", db="test")

cur=db.cursor()

before_t = time.time()

for f in range(0,int(num_iter)):

    id_num = ids[random.randrange(0,len(ids))]

    cur.execute("select ATTR2 from mytablename where ATTR1=%s" % (id_num))
    res=cur.fetchall()

after_t = time.time()

after_t = time.time()
print "MySQL time for %s: %s -- %s" % ( num_iter, after_t, before_t)
print "\t %s" % (after_t - before_t)
